#include <iostream.h>
#include <ctype.h>

const int NO_OF_LETTERS = 26;
const int DEFAULT_HEIGHT = 14;

struct Histogram {
public:
	Histogram();
	void reset(void);		// Reset counts to 0
	void add_character( const char );// Add ch to histogram
	void scale(const int=DEFAULT_HEIGHT);// Scale height
	void print(void) const;		// Display histogram
private:
	int the_letter_freq[NO_OF_LETTERS];// Stored letter freq
	int the_max_height;		// Maximum frequency
	int the_actual_max;		// Scale to
};


Histogram::Histogram()
{
	reset();
}

void Histogram::reset(void)
{
	for ( int ch_pos=0; ch_pos < NO_OF_LETTERS; ch_pos++ ) {
		the_letter_freq[ch_pos] = 0;
	}
	the_max_height = DEFAULT_HEIGHT; the_actual_max = 0;
}


void Histogram::add_character( const char character_to_add )
{
	char c = character_to_add;
	if ( islower(c) ) {
		c = toupper(c);
	}
	if ( isupper(c) ) {
		c = (char)( (int) c - (int) 'A'); // A-Z contiguous
		the_letter_freq[c]++;
	}
}

void Histogram::scale(const int height)
{
	int max=0,ch;
	the_max_height = height;

	for ( ch=0; ch<NO_OF_LETTERS; ch++ ) {
		if ( the_letter_freq[ch] > max ) {
			max = the_letter_freq[ch];
		}
	}

	the_actual_max = max;
	if ( max > 0 ) {
		for ( ch=0; ch<NO_OF_LETTERS; ch++ ) {
			int scaled = (the_letter_freq[ch]*height)/max;
			int rem    = (the_letter_freq[ch]*height)%max;
			the_letter_freq[ch] = (rem+rem>max?scaled+1 : scaled);
		}
	}
}

void Histogram::print(void) const
{
	for ( int j=the_max_height; j>0; j-- ) {
		cout << "  | ";
		for ( int ch_pos=0; ch_pos<NO_OF_LETTERS; ch_pos++) {
			if ( the_letter_freq[ch_pos] >= j )
				cout << "*";
			else
				cout << " ";
		}
		cout << " | " << "\n";
	}
	cout << "    --------------------------" << "\n";
	cout << "    ABCDEFGHIJKLMNOPQRSTUVWXYZ     * =  ";
	if (the_max_height > 0 )
		cout <<  (float) the_actual_max / (float) the_max_height;
	else
		cout << "0.0";
	cout << "\n";
}

void main()
{
	Histogram hist;
	char ch;

	while ( cin >> ch , !cin.eof() )
	{
		hist.add_character( ch );
	}
	hist.scale(12);
	hist.print();
}
