const int MAX_STRING = 80;

class String {
public:
	typedef const char * C_str;
	String();
	String( const char str[] );
	String( const char ch );
	int length(void) const;
	C_str c_string(void) const;
	friend String operator + ( const String& s1, const String& s2 );
	String mid( const int from , const int to ) const;
	friend Boolean operator == ( const String& s1, const String& s2 );
	friend Boolean operator != ( const String& s1, const String& s2 );
	friend Boolean operator >= ( const String& s1, const String& s2 );
	friend Boolean operator >  ( const String& s1, const String& s2 );
	friend Boolean operator <  ( const String& s1, const String& s2 );
	friend Boolean operator <= ( const String& s1, const String& s2 );
	friend ostream& operator << ( ostream& s, const String& str );
protected:
	int friend cmp( const String s1, const String& s2 );
private:
	char    the_text[MAX_STRING];	// Characters of the string
	int     the_length;	// Number of characters in the string
};

inline String::String()
{
	the_text[0] = '\0'; the_length = 0;
}

inline String::String( const char ch )
{
	the_text[0] = ch; the_text[1] = '\0'; the_length = 1;
}

String::String( const char str[] )
{
	int i=0;
	while ( (i < (MAX_STRING-1)) && (str[i] != '\0') )
	{
		the_text[i] = str[i]; i++;
	}
	the_text[i] = '\0'; the_length = i;
}

inline int String::length(void) const
{
	return the_length;
}

inline String::C_str String::c_string(void) const
{
	return the_text;
}

String operator + ( const String& s1, const String& s2 )
{
	String res;
	int i=0,j=s1.the_length;
	res = s1;
	while ( (j < MAX_STRING-1) && (s2.the_text[i] != '\0' ) )
	{
		res.the_text[j] = s2.the_text[i];
		j++; i++;
	}
	res.the_text[j] = '\0'; res.the_length = j;
	return res;
}

String String::mid( const int from , const int to ) const
{
	String res;
	int i;
	if ( from >= the_length || from < 0 || to < 0 || from > to )
		return res;
	i = from;
	while ( i <= to )
	{
		res.the_text[i-from] = the_text[i];
		i++;
	}
	res.the_text[i-from] = '\0'; res.the_length = to - from + 1;
	return res;
}

int cmp( const String s1, const String& s2 )
{
	int i=0;
	int min = ( s1.the_length < s2.the_length ?
				s1.the_length : s2.the_length );
	if ( s1. the_length != s2.the_length ) min++;	// Include '\0'
	while ( i < min )
	{
		int comp = (int) s1.the_text[i] - (int) s2.the_text[i];
		if ( comp != 0 )
			return comp;
		i++;
	}
	return 0;
}

inline Boolean operator == ( const String& s1, const String& s2 )
{
	return cmp(s1,s2) == 0 ? TRUE : FALSE;
}

inline Boolean operator != ( const String& s1, const String& s2 )
{
	return cmp(s1,s2) != 0 ? TRUE : FALSE;
}

inline Boolean operator >= ( const String& s1, const String& s2 )
{
	return cmp(s1,s2) >= 0 ? TRUE : FALSE;
}

inline Boolean operator >  ( const String& s1, const String& s2 )
{
	return cmp(s1,s2) >  0 ? TRUE : FALSE;
}

inline Boolean operator <  ( const String& s1, const String& s2 )
{
	return cmp(s1,s2) <  0 ? TRUE : FALSE;
}

inline Boolean operator <= ( const String& s1, const String& s2 )
{
	return cmp(s1,s2) <= 0 ? TRUE : FALSE;
}

ostream& operator << ( ostream& s, const String& str )
{
	s << str.the_text; return s;
}
