#include <iostream.h>
#include <iomanip.h>

typedef double T_Money;

#include "s50_bank.h"

void perform1(void);
void perform2(void);
void perform3(void);
void perform4(void);
void perform5(void);

void print_statement( Abstract_Account& );
void gift( Abstract_Account&, const T_Money );
void interest_at_end_of_day( Abstract_Account& );
void credit_interest( Abstract_Account& );

void main()
{
	const double DAILY_RATE = 0.00026116;	// 10% Annual rate
	const double DAILY_RATE_R2 = 0.00028596;// 11% Annual rate
	const double DAILY_RATE_R3 = 0.00031054;// 12% Annual rate

	I_Account::prelude( DAILY_RATE );

	S_Account::prelude( DAILY_RATE, DAILY_RATE_R2, DAILY_RATE_R3 );

	cout << setiosflags( ios::showpoint );
	cout << setiosflags( ios::fixed );

	cout << "\n" << "Test 4" << "\n";
	perform1();
	cout << "\n" << "Test 2" << "\n";
	perform2();
	cout << "\n" << "Test 3" << "\n";
	perform3();
	cout << "\n" << "Test 4" << "\n";
	perform4();
	cout << "\n" << "Test 5" << "\n";
	perform5();
}

void perform1()
{
	I_Account mike(100);
	S_Account corinna(1000);

	cout << setiosflags( ios::showpoint );
	cout << setiosflags( ios::fixed );

	print_statement( mike );
	gift( mike, 10 );
	interest_at_end_of_day( mike );
	credit_interest( mike );
	print_statement( mike );

	print_statement( corinna );
	gift( corinna, 100 );
	interest_at_end_of_day( corinna );
	credit_interest( corinna );
	print_statement( corinna );
}

void print_statement( Abstract_Account& customer )
{
	cout << "\n"; customer.type_of_account(); cout << "\n";
	customer.print_details(); cout << "\n";
}

void gift( Abstract_Account& customer, const T_Money amount )
{
	cout << "Deposit " << amount << "\n";
	customer.deposit(amount);
}

void interest_at_end_of_day( Abstract_Account& customer )
{
	cout << "Add Interest" << "\n";
	customer.calc_interest();
}

void credit_interest( Abstract_Account& customer )
{
	customer.add_interest();
}

void perform2()
{
	const int MAX_CUSTOMERS = 3;
	Abstract_Account *customers[MAX_CUSTOMERS];
	int i;

	customers[0] = new N_Account( 100 );
	customers[1] = new I_Account( 10000 );
	customers[2] = new S_Account( 25000 );

	for (i=0; i<MAX_CUSTOMERS; i++ )
	{
		customers[i]->calc_interest();
		customers[i]->add_interest();
	}


	for (i=0; i<MAX_CUSTOMERS; i++ )
	{
		cout << "Customer " << i << " :" << "\n";
		customers[i]->print_details();
		cout << "\n";
	}

	for (i=0; i<MAX_CUSTOMERS; i++ )
	{
		delete customers[i];
	}
}

void perform3()
{
	// Abstract_Account m;	// Will not compile
	S_Account mas(1000);
	mas.calc_interest();
}

void perform4()
{
	const int MAX_CUSTOMERS = 3;
	Abstract_Account *customers[MAX_CUSTOMERS];
	int i;

	N_Account normal(100);
	I_Account interest(8000);
	S_Account special_interest(25000);

	customers[0] = &normal;
	customers[1] = &interest;
	customers[2] = &special_interest;

	// At the end of each day the bank would calculate the interest
	//  if any to be credited to each account

	for (i=0; i<MAX_CUSTOMERS; i++ )
	{
		customers[i]->calc_interest();
		customers[i]->add_interest();
	}

	// To print mini statements for all customers

	for (i=0; i<MAX_CUSTOMERS; i++ )
	{
		cout << "Customer " << i << " :" << "\n";
		customers[i]->print_details();
		cout << "\n";
	}


}

void perform5()
{

	N_Account normal(100);
	I_Account interest(8000);
	S_Account special_interest(25000);
#ifdef BUG_BORLAND
	normal = interest;
	normal = special_interest;
#endif
}
