#include <iostream.h>
#include <stdlib.h>

template <class Type>
class Safe_array {
public:
	Safe_array( const int=10, const int=10 );
	~Safe_array();
	Type& operator()(const int, const int) const; // Access with checking
	Type& fast(const int,const int) const;	// Fast access
protected:
	void fail( const char mes[], const int, const int ) const;
private:
	int  the_no_rows;	// Number of the_no_rows
	int  the_no_columns;	// Number of the_no_columns
	Type **the_pp_cell;	// Pointer to array of ints
};

template <class Type>
Safe_array <Type>::Safe_array(const int rows, const int cols )
{
	the_no_rows = rows; the_no_columns = cols;

	if ( the_no_rows < 1 || the_no_columns < 1 )
		fail("Array dimensions invalid -" , rows , cols );

	the_pp_cell = new Type *[the_no_rows];
	if ( the_pp_cell == ( Type **) 0 )
		fail("Cannot create array" ,
			the_no_rows , the_no_columns );

	for (int i=0; i<the_no_rows; i++)
	{
		the_pp_cell[i] = new Type[the_no_columns];
		if ( the_pp_cell[i] == (Type * ) 0 )
		{
			fail("Cannot create array",
				the_no_rows, the_no_columns );
		}
	}
}

template <class Type>
Safe_array <Type>::~Safe_array()
{
	for (int i=0; i<the_no_rows; i++)
	{
		delete [] the_pp_cell[i];
	}
	delete [] the_pp_cell;
}

template <class Type>
Type& Safe_array <Type>::operator()( const int row, const int column) const
{
	if ( row<0 || row>=the_no_rows ||
	     column<0 || column>=the_no_columns )
		fail("Subscript overflow - subscripts ", row, column );
	return the_pp_cell[row][column];
}

template <class Type>
Type& Safe_array <Type>::fast( const int r , const int c) const
{
	return the_pp_cell[r][c];
}

template <class Type>
void Safe_array <Type>::fail(const char mes[], const int i,
		      const int j ) const
{
	cout << "\n" << "Error in class : Safe_array" << "\n";
	cout << mes << " " << i << " " << j << "\n";
	exit(-1);
}

void main1();
void main2();
void main3();

void main()
{
	main2();  main1(); main3();
}

void main1()
{
	const int the_no_rows = 3;
	const int cols = 4;
	Safe_array <int> table(the_no_rows,cols);
	int i,j;

	for( i=0; i<the_no_rows; i++ ) {
		for( j=0; j<cols; j++ ) {
			table(i,j) = i*10+j;
		}
	}

	cout << "\n";
	for( i=0; i<the_no_rows; i++ ) {
		for( j=0; j<cols; j++ ) {
			cout << table(i,j) << "\t";
		}
		cout << "\n";
	}

	cout << "\n";
	for( i=0; i<the_no_rows; i++ ) {
		for( j=0; j<cols; j++ ) {
			cout << table.fast(i,j) << "\t";
		}
		cout << "\n";
	}

	cout << "Using invalid subscript to access array" << "\n";
	table(the_no_rows,cols) = 2;
}

void main2()
{
	const int the_no_rows = 3;
	const int cols = 4;
	Safe_array <double> table(the_no_rows,cols);
	int i,j;

	for( i=0; i<the_no_rows; i++ ) {
		for( j=0; j<cols; j++ ) {
			table(i,j) = i*10+j + 0.5;
		}
	}

	cout << "\n";
	for( i=0; i<the_no_rows; i++ ) {
		for( j=0; j<cols; j++ ) {
			cout << table(i,j) << "\t";
		}
		cout << "\n";
	}

	cout << "\n";
	for( i=0; i<the_no_rows; i++ ) {
		for( j=0; j<cols; j++ ) {
			cout << table.fast(i,j) << "\t";
		}
		cout << "\n";
	}

}

void main3()
{
	Safe_array <char> table;
	Safe_array <int>  tab2(1);
}
