#include <stdlib.h>
#include "s99_type.h"

template <class Type>
struct Cell {			// Element of linked list
	Type    value;		// The value of the item in the set
	Cell <Type> *p_next;	// Pointer to next Cell
};

template <class Type>
struct Set_of_DESC {		// The Descriptor
	Cell <Type> *p_root;		// Root of list of values
	Cell <Type> *p_last;		// Last element
	int no_of_elements;	// Number of elements in list
	int references;		// references to this item
};

template <class Type>
class Set_of {
public:
	Set_of();			// Constructor empty
	Set_of( const Type );		// Constructor single element
	~Set_of();			// Destructor

	Set_of( Set_of <Type>& );	// copy constructor
	Set_of& operator = ( Set_of <Type>& ); // Assignment operator

	int cardinality() const;	// Number of items
	friend Set_of <Type> operator + (Set_of <Type>&, Set_of <Type>&);
	friend Boolean operator == (Set_of <Type>&, Set_of <Type>&);

	friend ostream& operator << ( ostream&, Set_of <Type>& );
protected:
	void first_element();		// Iterate set first element
	void next_element();		// Iterate move to next
	Type current_element() const;	// Iterate return current item
	int is_at_end() const;		// Iterate at end
	void add_at_end( const Type );	// Add to list at end

	void release_storage( void );	// Release sets storage
	void fail( const char[] ) const;// Give up
private:

	Set_of_DESC <Type> *the_p_desc;	// Description of set
	Cell <Type> *the_p_current;		// Current element looked at
};

template <class Type>
Set_of <Type>::Set_of()
{
	the_p_desc = new Set_of_DESC <Type>[1];
	if ( the_p_desc == (Set_of_DESC <Type>*) 0 )
		fail("Cannot create Set_of - descriptor");
	the_p_desc->no_of_elements = 0;
	the_p_desc->references = 1;
	the_p_desc->p_root = (Cell <Type>*) 0;
	the_p_desc->p_last = (Cell <Type>*) 0;
}

template <class Type>
Set_of <Type>::Set_of(const Type item)
{
	the_p_desc = new Set_of_DESC <Type>[1];
	if ( the_p_desc == (Set_of_DESC <Type>*) 0 )
		fail("Cannot create Set_of - descriptor");
	the_p_desc->no_of_elements = 1;
	the_p_desc->references = 1;
	the_p_desc->p_root = new Cell <Type>[1];
	if ( the_p_desc->p_root == (Cell <Type>*) 0 )
		fail("Cannot create Cell <Type> - descriptor");
	the_p_desc->p_last = the_p_desc->p_root;
	the_p_desc->p_root->value = item;
	the_p_desc->p_root->p_next = (Cell <Type>*) 0;
}

template <class Type>
void Set_of <Type>::release_storage(void)
{
	if (--(the_p_desc->references)==0)
	{
		Cell <Type> *p_active = the_p_desc->p_root;
		while ( p_active != (Cell <Type>*) 0 )
		{
			Cell <Type> *p_remove = p_active;
			p_active = p_active->p_next;
			delete [] p_remove;
		}
		delete [] the_p_desc;
	}
}

template <class Type>
void Set_of <Type>::fail(const char mes[]) const
{
	cout << "\n" << "Error in class : Set_of" << "\n";
	cout << mes << "\n";
	exit(-1);
}

template <class Type>
Set_of <Type>::~Set_of()
{
	release_storage();
}


template <class Type>
Set_of <Type>::Set_of(Set_of <Type>& copy)
{
	copy.the_p_desc->references++;	// Copied item reference count
	the_p_desc = copy.the_p_desc;	// Copy the pointer to the descriptor
	the_p_current = copy.the_p_desc->p_root;// Current element
}

template <class Type>
int Set_of <Type>::cardinality(void) const
{
	return the_p_desc->no_of_elements;
}

template <class Type>
void Set_of <Type>::first_element(void)
{
	the_p_current = the_p_desc->p_root;
}

template <class Type>
void Set_of <Type>::next_element(void)
{
	if ( the_p_current != (Cell <Type>*) 0 )
		the_p_current = the_p_current->p_next;
}


template <class Type>
Type Set_of <Type>::current_element(void) const
{
	if (the_p_current == (Cell <Type>*) 0 )
	{
		fail("Selecting item beyond end");
	}
	return the_p_current->value;
}

template <class Type>
int Set_of <Type>::is_at_end(void) const
{
	return the_p_current == (Cell <Type>*) 0;
}

template <class Type>
void Set_of <Type>::add_at_end( const Type item )
{
	Cell <Type> *p_add = new Cell <Type> [1];
	if ( p_add == (Cell <Type>*) 0 )
		fail("Cannot create new element");
	p_add->value = item; p_add->p_next = (Cell <Type>*) 0;
	if (the_p_desc->p_root == (Cell <Type>*) 0 )
	{
		the_p_desc->p_root = p_add;
		the_p_desc->p_last = p_add;
	} else {
		the_p_desc->p_last->p_next = p_add;
		the_p_desc->p_last = p_add;
	}
	the_p_desc->no_of_elements++;
}


template <class Type>
Set_of <Type>& Set_of <Type>::operator = ( Set_of <Type>& set1 )
{
	set1.the_p_desc->references++;	// Copied items ref count
	release_storage();		// Storage for overwritten item
	the_p_desc = set1.the_p_desc;
	return *this;
}

template <class Type>
Set_of <Type> operator + ( Set_of <Type>& set1, Set_of <Type>& set2 )
{
	Set_of <Type> result;

	set1.first_element(); set2.first_element();

	while ( !set1.is_at_end() || !set2.is_at_end() )
	{
		if ( set1.is_at_end() )
		{
			result.add_at_end( set2.current_element() );
			set2.next_element();
		} else
		if ( set2.is_at_end() )
		{
			result.add_at_end( set1.current_element() );
			set1.next_element();
		} else
		if ( set1.current_element() < set2.current_element() )
		{
			result.add_at_end(set1.current_element());
			set1.next_element();
		} else
		if ( set1.current_element() > set2.current_element() )
		{
			result.add_at_end( set2.current_element() );
			set2.next_element();
		} else
		if ( set1.current_element() == set2.current_element() )
		{
			result.add_at_end( set1.current_element() );
			set1.next_element();
			set2.next_element();
		}
	}
	return result;
}


template <class Type>
Boolean operator == ( Set_of <Type>& set1, Set_of <Type>& set2 )
{
	set1.first_element(); set2.first_element();
	if ( set1.cardinality() != set2.cardinality() ) return FALSE;
	if ( set1.cardinality() == 0 ) return TRUE;
	while ( !set1.is_at_end() )
	{
		if ( set1.current_element() != set2.current_element() )
		{
			return FALSE;
		}
		set1.next_element(); set2.next_element();
	}
	return TRUE;
}


template <class Type>
ostream& operator << ( ostream& s, Set_of <Type>& set1 )
{
	int i;
	set1.first_element(); s << "(";
	for( i=0; i < set1.cardinality(); i++ )
	{
		s << set1.current_element();
		set1.next_element();
		if ( i != set1.cardinality()-1 ) s << ",";
	}
	s << ")";
	return s;
}
