#include <iostream.h>
#include "s99_type.h"

template <class Type>
class A_Bag
{
public:
	virtual int items_in() = 0;		// Items in Bag
	virtual int holds( const Type& ) = 0;	// Contained in bag
	virtual void add( const Type& ) = 0;	// Add an item
	virtual ~A_Bag() {}			// Destructor
};

template <class Type>
class A_It_Bag
{
public:
	virtual void reset() = 0;	// Reset to start
	virtual void operator ++(int) = 0;// Next item
	virtual operator int () = 0;	// Is at end
	virtual Type current() = 0;	// Return current item
	virtual ~A_It_Bag() {}		// Destructor
};

// -------------------------------------------------------------------------


template <class Type>
class List
{
public:
	List();			// Constructor for list
	~List();		// Destructor release list storage
	void add( const Type& );// Add to start of list
private:
	struct Node { Type item; Node *next; };
	typedef Node *P_Node;
	friend It_List <Type>;	// So Iterator can access elements

	P_Node the_head;	// The head of the list
};

template <class Type>
List<Type>::List()
{
	the_head = (Node*) 0;	// Set to empty
}

template <class Type>
List<Type>::~List()
{
	P_Node p = the_head;	// Loop releasing storage
	while ( p != (P_Node) 0 )
	{
		P_Node temp = p;
		p = p->next;
		delete [] temp;	// for the list
	}
}

template <class Type>
void List<Type>::add( const Type& to_add )
{
	Node *temp = new Node[1];// new node
	temp->item = to_add;	// add item
	temp->next = the_head;	// chain rest of list to node
	the_head = temp;	// new head is at new node
}

// -------------------------------------------------------------------------

template <class Type>
class It_List
{
public:
	It_List();			// Create unattached iterator
	It_List( List <Type>& );	// Attached iterator to list
	void reset();			// reset to start of list
	void operator ++(int);		// next item in list
	operator int();			// at end of list
	Type current();			// current item in list
private:
	List <Type>::Node*  the_current;// Current node
	List <Type>*        the_ins_list;// List associated with
};

template <class Type>
It_List<Type>::It_List(List <Type>& list)
{
	the_current = (List <Type>::Node *) 0;
	the_ins_list = &list;		// Set up iterator for list
	reset();			// set to first element
}

template <class Type>
It_List<Type>::It_List()
{
	the_current = (List <Type>::Node *) 0;
	the_ins_list = (List <Type> *) 0;	// Set up null iterator
}

template <class Type>
void It_List <Type>::reset()
{
	the_current =  the_ins_list != ( List <Type> *) 0	// Set
			       ? the_ins_list->the_head		// To start
			       : (List <Type>::Node * ) 0;	// Empty
}

template <class Type>
void It_List <Type>::operator ++(int)
{
	if ( the_current != (List <Type>::Node *) 0 )
		the_current = the_current->next;
}


template <class Type>
It_List <Type>::operator int()
{
	return ( the_current != (List <Type>::Node *) 0 );
}

template <class Type>
Type It_List <Type>::current()
{
	return the_current->item;
}

// -------------------------------------------------------------------------

template <class Type>
class Bag : public A_Bag <Type>, public List <Type>
{
public:
	Bag();			// The constructor
	int items_in();		// Return number of items in bag
	int holds( const Type& ); // Contained in bag
	void add( const Type& );// Add an item to the bag
private:
	int the_number_in;	// The number of items in the bag
};

template <class Type>
Bag <Type>::Bag() : List <Type>()// Call the list constructor
{
	the_number_in = 0;	// Set number in Bag to 0
}

template <class Type>
int Bag<Type>::items_in()
{
	return the_number_in;	// Number in bag
}

template <class Type>
int Bag<Type>::holds( const Type& item )
{
	int number_of = 0;
	for( It_Bag <Type> iter( *this ); iter; iter++ )
	{
		if ( item == iter.current() ) number_of++;
	}
	return number_of;
}


template <class Type>
void Bag<Type>::add( const Type& to_add )
{
	the_number_in++;
	List <Type>::add( to_add );// Use list to store
}

template <class Type>
class It_Bag :  public A_It_Bag <Type>, public It_List <Type>
{
public:
	It_Bag( Bag <Type>& );	// Iterator for Bag ...
	It_Bag();		// Un assigned
	void reset();		// Reset to start
	void operator ++(int);	// Next item
	operator int ();	// Is at end
	Type current();		// Return current item
};

template <class Type>
It_Bag <Type>::It_Bag( Bag <Type>& b) : It_List <Type> ( b )
{
}

template <class Type>
It_Bag <Type>::It_Bag() : It_List <Type>()
{
}

template <class Type>
void It_Bag <Type>::reset()
{
	It_List<Type>::reset();
}

template <class Type>
void It_Bag <Type>::operator ++(int)
{
	(*this).It_List <Type>::operator++(1);
}


template <class Type>
It_Bag <Type>::operator int()
{
	return  (*this).It_List <Type>::operator int();
}

template <class Type>
Type It_Bag <Type>::current()
{
	return It_List<Type>::current();
}


void main1();  void main2();   void main3();   void main4();
void main00(); void main01();  void main5();

void main()
{
	cout << "case 00 of use of Container Iterator" << "\n";
	main00(); cout << "\n";
	cout << "case 01 of use of Container Iterator" << "\n";
	main01(); cout << "\n";
	cout << "case 1 of use of Container Iterator" << "\n";
	main1(); cout << "\n";
	cout << "case 2 of use of Container Iterator" << "\n";
	main2(); cout << "\n";
	cout << "case 3 of use of Container Iterator" << "\n";
	main3(); cout << "\n";
	cout << "case 3 of use of Container Iterator" << "\n";
	main4(); cout << "\n";
	cout << "case 4 of use of Container Iterator" << "\n";
	main5(); cout << "\n";
}

void main00()
{
	const int MAX = 3;
	int numbers[ MAX ];

	numbers[0] = 3; numbers[1] = 1; numbers[2] = 4;

	for ( int iter=0; iter<MAX; iter++ )
	{
		cout << numbers[ iter ] << "\t";
	}
}

void main01()
{
	Bag <int> numbers;

	numbers.add(3); numbers.add(1); numbers.add(4);

	for( It_Bag <int> int_iter(numbers); int_iter; int_iter++ )
	{
		cout << int_iter.current() << "\t";
	}
}


void main1()
{
	Bag <int> numbers;
	Bag <char*> strings;

	numbers.add(1); numbers.add(2); numbers.add(3);

	for( It_Bag <int> int_iter(numbers); int_iter; int_iter++ )
	{
		cout << int_iter.current() << "\t";
	}
	cout << "\n";

	cout << "Bag contains 2 ";
	cout << ( numbers.holds(2) ? "Yes" : "No" ) << "\n";

	cout << "Bag contains 4 ";
	cout << ( numbers.holds(4) ? "Yes" : "No" ) << "\n";


	strings.add("One"); strings.add("Two"); strings.add("Three");

	for( It_Bag <char *> str_itr(strings); str_itr; str_itr++ )
	{
		cout << str_itr.current() << "\t";
	}
	cout << "\n";
}

// Will not work with prototype
//template <class Type> void print(Bag<Type>& );


template <class Type>
void print( Bag <Type>& c )
{
	It_Bag <Type> itt(c);		// Iterator for c
	for( itt.reset(); itt; itt++ )
	{
		cout << itt.current() << "\t";
	}
	cout << "\n";
}

void main2()
{
	Bag <int> numbers;
	Bag <char *> strings;

	numbers.add(10); numbers.add(20); numbers.add(30);
	strings.add("dix"); strings.add("onze"); strings.add("douze");

	print( numbers );
	print( strings );
}

void main3()
{
	Bag <char> characters;		// Bag for characters
	It_Bag <char> outer(characters);// Iterator for characters
	It_Bag <char> inner;		// Unassigned char iterator

	cout << "" << "\n";

	characters.add('A'); characters.add('B');
	characters.add('C'); characters.add('D');

	cout << "Items in bag = " << characters.items_in() << "\n";
	for( outer.reset(); outer; outer++ )
	{
		cout << "Items remaining in bag = < ";
		for( inner = outer; inner; inner++ )
		{
			cout << inner.current() << " ";
		}
		cout << ">" << "\n";
	}
}

template <class Type>
void print2( A_It_Bag <Type>& item )
{
	for( item.reset(); item; item++ )
	{
		cout << item.current() << "\t";
	}
	cout << "\n";
}

void main4()
{
	Bag <int> num;
	A_Bag <int> *numbers = &num;

	Bag <char *> str;
	A_Bag <char *> *strings = &str;

	It_Bag <int> it(num);
	A_It_Bag <int> *itt = &it;

	It_Bag <char *> s_it(str);
	A_It_Bag <char *> *s_itt = &s_it;

	numbers->add(1); numbers->add(4); numbers->add(9);
	strings->add("red"); strings->add("white"); strings->add("blue");

	cout << "Using Polymorpsium" << "\n";

	print2( *itt );
	print2( *s_itt );
}


#ifdef BORLAND

#include "s60_set3.h"

template <class Type>
void print_set( Set_of <Type>& set)
{
	cout << "Set contains : " << set << "\n";
}

/*
 *	Bag < Bag <int> > bag_of_bags;
 *	The only problem with this code is that a copy is made of
 *	 of the bag when putting it into the bag of bags.
 *	The storage will then be released twice!!
 *	 Need to use Descriptors for implementation of List
 */

extern "C" void abort();

void main5()
{
	// This will fail to compile if the next line is further down
	// This MUST be a BUG
	Set_of <int>		set_1,set_2;
	Bag < Set_of <int> >	bag_of_sets;
	It_Bag < Set_of <int> > it_bag_of_sets( bag_of_sets );

	// Place some numbers in the sets

	set_1 = Set_of <int> (1) + Set_of <int> (2) +  Set_of <int> (3);
	set_2 = Set_of <int> (10) + Set_of <int> (20);

	// Add the sets to the bag

	bag_of_sets.add( set_1 );
	bag_of_sets.add( set_2 );

	// Now print the contents of the bag

	for( it_bag_of_sets.reset(); it_bag_of_sets; it_bag_of_sets++ )
	{
		cout << "Set contains : ";
		cout << it_bag_of_sets.current();
		cout << "\n";
	}
}
#else
void main5() { }
#endif
