#include <iostream.h>
#include "s99_type.h"

template <class Type>
class A_Bag
{
public:
	virtual int items_in() = 0;		// Items in Bag
	virtual int holds( const Type& ) = 0;	// Contained in bag
	virtual void add( const Type& ) = 0;	// Add an item
	virtual ~A_Bag() {}			// Destructor
};

template <class Type>
class A_It_Bag
{
public:
	virtual void reset() = 0;	// Reset to start
	virtual void operator ++(int) = 0;// Next item
	virtual operator int () = 0;	// Is at end
	virtual Type current() = 0;	// Return current item
	virtual ~A_It_Bag() {}		// Destructor
};

// -------------------------------------------------------------------------
#ifdef BORLAND
template <class Type>
class List
{
public:
	List();			// Constructor for list
	~List();		// Destructor release list storage
	void add( const Type& );// Add to start of list
private:
	struct Node { Type item; Node *next; };
	typedef Node *P_Node;
	friend It_List <Type>;	// So Iterator can access elements

	P_Node the_head;	// The head of the list
};

template <class Type>
List<Type>::List()
{
	the_head = (Node*) 0;	// Set to empty
}

template <class Type>
List<Type>::~List()
{
	P_Node p = the_head;	// Loop releasing storage
	while ( p != (P_Node) 0 )
	{

		P_Node temp = p;
		p = p->next;
		delete [] temp;	// for the list
	}
}

template <class Type>
void List<Type>::add( const Type& to_add )
{
	Node *temp = new Node[1];// new node
	temp->item = to_add;	// add item
	temp->next = the_head;	// chain rest of list to node
	the_head = temp;	// new head is at new node
}

// -------------------------------------------------------------------------

template <class Type>
class It_List
{
public:
	It_List();			// Create unattached iterator
	It_List( List <Type>& );	// Attached iterator to list
	void reset();			// reset to start of list
	void operator ++(int);		// next item in list
	operator int();			// at end of list
	Type current();			// current item in list
private:
	List <Type>::Node*  the_current;// Current node
	List <Type>*        the_ins_list;// List associated with
};

template <class Type>
It_List<Type>::It_List(List <Type>& list)
{
	the_current = (List <Type>::Node *) 0;
	the_ins_list = &list;		// Set up iterator for list
	reset();			// set to first element
}

template <class Type>
It_List<Type>::It_List()
{
	the_current = (List <Type>::Node *) 0;
	the_ins_list = (List <Type> *) 0;	// Set up null iterator
}

template <class Type>
void It_List <Type>::reset()
{
	the_current =  the_ins_list != ( List <Type> *) 0	// Set
			       ? the_ins_list->the_head		// To start
			       : (List <Type>::Node * ) 0;	// Empty
}

template <class Type>
void It_List <Type>::operator ++(int)
{
	if ( the_current != (List <Type>::Node *) 0 )
		the_current = the_current->next;
}


template <class Type>
It_List <Type>::operator int()
{
	return ( the_current != (List <Type>::Node *) 0 );
}

template <class Type>
Type It_List <Type>::current()
{
	return the_current->item;
}
#else
#include "s65_adtf.cpp"
#endif
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------

template <class Type, class Based_on,class It_based_on>
class Bag : public A_Bag <Type>, public Based_on
{
public:
	Bag();			// The constructor
	int items_in();		// Return number of items in bag
	int holds( const Type& ); // Contained in bag
	void add( const Type& );// Add an item to the bag
private:
	int the_number_in;	// The number of items in the bag
};

template <class Type,class Based_on,class It_Based_on>
Bag <Type,Based_on,It_Based_on>::Bag() : Based_on()// Call the constructor
{
	the_number_in = 0;	// Set number in Bag to 0
}

template <class Type,class Based_on,class It_Based_on>
int Bag<Type,Based_on,It_Based_on>::items_in()
{
	return the_number_in;	// Number in bag
}

template <class Type,class Based_on,class It_based_on>
int Bag<Type,Based_on,It_based_on>::holds( const Type& item )
{
	int number_of = 0;
	for( It_Bag <Type,Based_on,It_based_on> iter( *this ); iter; iter++ )
	{
		if ( item == iter.current() ) number_of++;
	}
	return number_of;
}


template <class Type,class Based_on,class It_Based_on>
void Bag<Type,Based_on,It_Based_on>::add( const Type& to_add )
{
	the_number_in++;
	Based_on::add( to_add );// Use list to store
}

// -------------------------------------------------------------------------

template <class Type, class Based_on, class It_Based_on>
class It_Bag :  public A_It_Bag <Type>, public It_Based_on
{
public:
	It_Bag( Bag <Type,Based_on,It_Based_on>& );	// Iterator for Bag ...
	It_Bag();		// Un assigned
	void reset();		// Reset to start
	void operator ++(int);	// Next item
	operator int ();	// Is at end
	Type current();		// Return current item
};

template <class Type, class Based_on, class It_Based_on>
It_Bag <Type, Based_on, It_Based_on>::It_Bag( Bag <Type,Based_on,It_Based_on>& b) :
			It_Based_on( b )
{
}

template <class Type,class Based_on,class It_Based_on>
It_Bag <Type,Based_on,It_Based_on>::It_Bag() : It_Based_on()
{
}

template <class Type,class Based_on,class It_Based_on>
void It_Bag <Type,Based_on,It_Based_on>::reset()
{
	It_Based_on::reset();
}

template <class Type,class Based_on,class It_Based_on>
void It_Bag <Type,Based_on,It_Based_on>::operator ++(int)
{
	(*this).It_Based_on::operator++(1);
}


template <class Type,class Based_on,class It_Based_on>
It_Bag <Type,Based_on,It_Based_on>::operator int()
{
	return  (*this).It_Based_on::operator int();
}

template <class Type,class Based_on,class It_Based_on>
Type It_Bag <Type,Based_on,It_Based_on>::current()
{
	return It_Based_on::current();
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------

template <class Type>
class Array
{
public:
	Array();		// Constructor for array
	~Array();		// Destructor
	void add( const Type& );// Add to array
private:
	friend It_Array <Type>;	// So Iterator can access elements
	int	the_size;
	int	the_last;
	Type	*the_vec;	// The elements held
};

template <class Type>
Array<Type>::Array()
{
	the_size = 100;
	the_vec = new Type[the_size];
	the_last = -1;
}

template <class Type>
Array<Type>::~Array()
{
	delete [] the_vec;
}

template <class Type>
void Array<Type>::add( const Type& to_add )
{
	if ( the_last < the_size )
	{
		the_vec[++the_last] = to_add;
	}
}

// -------------------------------------------------------------------------

template <class Type>
class It_Array
{
public:
	It_Array();			// Create unattached iterator
	It_Array( Array <Type>& );	// Attached iterator to array
	void reset();			// reset to start of array
	void operator ++(int);		// next item in array
	operator int();			// at end of array
	Type current();			// current item in array
private:
	int	the_current;		// Current element
	Boolean	the_end;		// Reached the end
	Array <Type>*	the_p_array;	// Array associated with
};

template <class Type>
It_Array<Type>::It_Array(Array <Type>& array)
{
	the_p_array = &array;		// Set up iterator for array
	reset();			// set to first element
}

template <class Type>
It_Array<Type>::It_Array()
{
	the_p_array = (Array <Type>*) 0;// Set up null iterator
	the_end     = TRUE;
}

template <class Type>
void It_Array <Type>::reset()
{
	the_current = 0;
	the_end = TRUE;
	if ( the_p_array != (Array <Type> *) 0 )
	{
		if ( the_p_array->the_last >= 0 )
			the_end = FALSE;
	}
}

template <class Type>
void It_Array <Type>::operator ++(int)
{
	if ( the_current < the_p_array->the_last )
		the_current++;
	else
		the_end = TRUE;
}

template <class Type>
It_Array <Type>::operator int()
{
	return !the_end;
}

template <class Type>
Type It_Array <Type>::current()
{
	return the_p_array->the_vec[the_current];
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------

void main1();  void main2();   void main3();   void main4();
void main00(); void main01();  void main02();  void main5();

void main()
{
	cout << "case 00 of use of a C++ vec" << "\n";
	main00(); cout << "\n";
	cout << "case 01 of use of Bag based on List" << "\n";
	main01(); cout << "\n";
	cout << "case 02 of use of Bag based on Array" << "\n";
	main02(); cout << "\n";
	cout << "case 1 of use of Container Iterator" << "\n";
	main1(); cout << "\n";
	cout << "case 2 of use of Container Iterator" << "\n";
	main2(); cout << "\n";
	cout << "case 3 of use of Container Iterator" << "\n";
	main3(); cout << "\n";
	cout << "case 4 of use of Container Iterator" << "\n";
	main4(); cout << "\n";
	cout << "case 5 of use of Container Iterator" << "\n";
	main5(); cout << "\n";
}

void main00()
{
	const int MAX = 3;
	int numbers[ MAX ];

	numbers[0] = 3; numbers[1] = 1; numbers[2] = 4;

	for ( int iter=0; iter<MAX; iter++ )
	{
		cout << numbers[ iter ] << "\t";
	}
}

void main01()
{
	Bag <int, List<int> , It_List <int> > numbers;

	numbers.add(3); numbers.add(1); numbers.add(4);

	It_Bag <int, List<int> , It_List<int> > int_iter(numbers);
	for( int_iter.reset(); int_iter; int_iter++ )
	{
		cout << int_iter.current() << "\t";
	}
}

void main02()
{
	Bag <int, Array<int> , It_Array<int> > numbers;

	numbers.add(3); numbers.add(1); numbers.add(4);

	It_Bag<int, Array<int>, It_Array<int> > int_iter(numbers);
	for( int_iter.reset(); int_iter; int_iter++ )
	{
		cout << int_iter.current() << "\t";
	}
}

typedef Bag    < int, List<int>, It_List<int> >  Bag_int;
typedef It_Bag < int, List<int>, It_List<int> >  It_Bag_int;
typedef Bag    < int, Array<int>, It_Array<int> >  Bag_int_a;
typedef It_Bag < int, Array<int>, It_Array<int> >  It_Bag_int_a;
typedef Bag    < char*, List<char*>, It_List<char*> > Bag_str;
typedef It_Bag < char*, List<char*>, It_List<char*> > It_Bag_str;

void main1()
{
	Bag_int numbers;
	Bag_str strings;

	numbers.add(1); numbers.add(2); numbers.add(3);

	for( It_Bag_int int_iter(numbers); int_iter; int_iter++ )
	{
		cout << int_iter.current() << "\t";
	}
	cout << "\n";

	cout << "Bag contains 2 ";
	cout << ( numbers.holds(2) ? "Yes" : "No" ) << "\n";

	cout << "Bag contains 4 ";
	cout << ( numbers.holds(4) ? "Yes" : "No" ) << "\n";


	strings.add("One"); strings.add("Two"); strings.add("Three");

	for( It_Bag_str str_itr(strings); str_itr; str_itr++ )
	{
		cout << str_itr.current() << "\t";
	}
	cout << "\n";
}

// Will not work with prototype
//template <class Type> void print(Bag<Type>& );


template <class Type, class Based_on, class It_Based_on >
void print( Bag <Type,Based_on,It_Based_on>& c )
{
	It_Bag <Type,Based_on,It_Based_on> itt(c);   // Iterator for c
	for( itt.reset(); itt; itt++ )
	{
		cout << itt.current() << "\t";
	}
	cout << "\n";
}

void main2()
{
	Bag_int numbers;
	Bag_str strings;

	numbers.add(10); numbers.add(20); numbers.add(30);
	strings.add("dix"); strings.add("onze"); strings.add("douze");

	print( numbers );
	print( strings );
}

typedef Bag    < char, List<char>, It_List<char> > Bag_char;
typedef It_Bag < char, List<char>, It_List<char> > It_Bag_char;

void main3()
{
	Bag_char	characters;	// Bag for characters
	It_Bag_char	outer(characters);// Iterator for characters
	It_Bag_char	inner;		// Unassigned char iterator

	cout << "" << "\n";

	characters.add('A'); characters.add('B');
	characters.add('C'); characters.add('D');

	cout << "Items in bag = " << characters.items_in() << "\n";
	for( outer.reset(); outer; outer++ )
	{
		cout << "Items remaining in bag = < ";
		for( inner = outer; inner; inner++ )
		{
			cout << inner.current() << " ";
		}
		cout << ">" << "\n";
	}
}

void process_any(  A_It_Bag <int>& );
void process_any( A_It_Bag <int>& it_a_bag )
{
	cout << "Bag contains [ ";
	for (it_a_bag.reset(); it_a_bag; it_a_bag++ )
	{
		cout << it_a_bag.current() << " ";
	}
	cout << "]" << "\n";
}

// Note the need to use the operator syntax
//	Otherwise the wrong operation will be performed

void process_any_2( A_It_Bag <int>* );
void process_any_2( A_It_Bag <int>* it_a_bag )
{
	cout << "Bag contains [ ";
	for (it_a_bag->reset(); it_a_bag->operator int();
				it_a_bag->operator ++(1) )
	{
		cout << it_a_bag->current() << " ";
	}
	cout << "]" << "\n";
}


void main4()
{
	Bag_int		bag_of_int;
	It_Bag_int	it_bag_of_int( bag_of_int );
	Bag_int_a	bag_of_int_a;
	It_Bag_int_a	it_bag_of_int_a( bag_of_int_a );

	bag_of_int.add(1);    bag_of_int.add(2);    bag_of_int.add(3);
	bag_of_int_a.add(10); bag_of_int_a.add(20); bag_of_int_a.add(30);

	process_any( it_bag_of_int_a );
	process_any( it_bag_of_int );

	process_any_2( &it_bag_of_int_a );
	process_any_2( &it_bag_of_int );
}


#ifdef BORLAND


#include "s60_set3.h"

typedef Set_of <int> Set_of_int;

typedef Bag    < Set_of_int, List< Set_of_int >,
		 It_List< Set_of_int > > Bag_of_set_of_int;

typedef It_Bag < Set_of_int, List< Set_of_int >,
		 It_List< Set_of_int > > It_Bag_of_set_of_int;

template <class Type>
void print_set( Set_of <Type>& set)
{
	cout << "Set contains : " << set << "\n";
}

/*
 *	Bag < Bag <int> > bag_of_bags;
 *	The only problem with this code is that a copy is made of
 *	 of the bag when putting it into the bag of bags.
 *	The storage will then be released twice!!
 *	 Need to use Descriptors for implementation of List
 */

extern "C" void abort();

void main5()
{
	// This will fail to compile if the next line is further down
	// This MUST be a BUG
	Set_of <int>		set_1,set_2;
	Bag_of_set_of_int	bag_of_sets;
	It_Bag_of_set_of_int	it_bag_of_sets( bag_of_sets );

	// Place some numbers in the sets

	set_1 = Set_of <int> (1) + Set_of <int> (2) +  Set_of <int> (3);
	set_2 = Set_of <int> (10) + Set_of <int> (20);

	// Add the sets to the bag

	bag_of_sets.add( set_1 );
	bag_of_sets.add( set_2 );

	// Now print the contents of the bag

	for( it_bag_of_sets.reset(); it_bag_of_sets; it_bag_of_sets++ )
	{
		cout << "Set contains : ";
		cout << it_bag_of_sets.current();
		cout << "\n";
	}
}
#else
void main5() { }
#endif

