#include <iostream.h>
#include <stdlib.h>

#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>
#include <string.h>

#include "s10_a.h" 			// Account
#include "s99_type.h"			// Boolean

#ifdef BORLAND
#include "s55_vec4.h"			// String Vec
#else
#include "s55_vec3.h" 			// String Vec not nested class
#endif
#include "s39_str.h"			// String class

typedef double T_Money;

#include "s50_bank.h"			// I_Account S_I_Account

typedef const char * C_str;		// C++ string
typedef char * C_w_str;			// C++ writable string


//	The base of all objects that are to made persistant
//	methods
//		new_object	Make a new copy with stored data values
//		portrait	[Name of class: Data in the classs]

class Persistent
{
public:
	virtual Persistent* new_object( C_str ) = 0;	// Clone with state
	virtual C_w_str portrait(C_w_str, const int) const = 0;	// Desc
	virtual ~Persistent() {};			// Destructor
};

// -----------------------------------------------------------------------

//	Persistent creator
//	Creates objects from stored string portraits


class Object_maker
{
public:
	static void record(C_str, Persistent* );	// Record
	static Persistent * make_a( C_str, C_str ); 	// Make an instance
	enum { SIZE = 17 };				// Hash table size
private:
	static  String_vec <Persistent*> the_examples;	// Stored examples
};


// The stored paterns of objects have global life time

String_vec <Persistent*> Object_maker::the_examples(Object_maker::SIZE);

void Object_maker::record( C_str class_name, Persistent* object)
{
	the_examples( class_name ) = object;
}

Persistent * Object_maker::make_a( C_str class_name, C_str state )
{
	return the_examples(class_name)->new_object(state);
}

#ifdef Nasty_Bodge
int map( const char name[] );

int map( const char name[] )
{
	switch ( name[0] )
	{
		case 'F' : return 0;
		case 'I' : return 1;
		case 'P' : return 2;
	}
        return 0;
}

class Object_maker
{
public:
	static void record(C_str, Persistent* );	// Record
	static Persistent * make_a( C_str, C_str ); 	// Make an instance
	enum { SIZE = 17 };				// Hash table size
private:
	static  Safe_vec <Persistent*> the_examples;	// Stored examples
};


// The stored paterns of objects have global life time

Safe_vec <Persistent*> Object_maker::the_examples(Object_maker::SIZE);

void Object_maker::record( C_str class_name, Persistent* object)
{
	the_examples( map(class_name) ) = object;
}

Persistent * Object_maker::make_a( C_str class_name, C_str state )
{
	return the_examples(map(class_name) )->new_object(state);
}
#endif
// -----------------------------------------------------------------------

class Int : public Persistent
{
public:
	Int( const int = 0 );			// constructor
	Int( C_str );				// Recreate with saved values
	Persistent *new_object( C_str );	// Generate a new copy
	C_w_str portrait(C_w_str,const int) const;// class and contents
	operator int() const;			// Conversion operator
private:
	int	the_storage; 			// Integer
};

Int::Int(const int n)
{
	the_storage = n;		// Conventional constructor
}

Int::Int( C_str str )
{					// Recreate object data values
	the_storage = atoi( str );	// which were saved as a string
}

Persistent* Int::new_object( C_str str )
{
	return new Int(str);		// new_object with new state
}

C_w_str Int::portrait( C_w_str desc, const int length ) const
{
	ostrstream text(desc,length);	// Write object name and value
	text << "[Int:  " << the_storage << "]" << '\0';
	return desc;
}

Int::operator int() const
{
	return the_storage;		// Conversion operator
}

// -----------------------------------------------------------------------

class Float : public Persistent
{
public:
	Float( const float = 0.0 );		// constructor
	Float( C_str );				// Recreate with saved values
	Persistent *new_object( C_str );	// Generate a new copy
	C_w_str portrait(C_w_str,const int) const;// class and contents
	operator float() const;			// Conversion operator
private:
	float	the_storage;			// Float
};

Float::Float(const float n)
{
	the_storage = n;		// Conventional constructor
}

Float::Float( C_str str )
{					// Recreate object data values
	the_storage = atof( str );	// which were saved as a string
}

Persistent* Float::new_object( C_str str )
{
	return new Float(str);		// new_object with new state
}

C_w_str Float::portrait( C_w_str desc, const int length ) const
{
	// deliver as a C++ string the object name and value
	ostrstream text(desc,length);
	text << "[Float:  " << the_storage << "]" << '\0';
	return desc;
}

Float::operator float() const
{
	return the_storage;		// Conversion operator
}


// -----------------------------------------------------------------------


void main1(); void main2(); void main3(); void main4(); void main5();

void main()
{
	main1(); main2(); main3(); main4(); main5();
}

void what_is( Persistent& );
void what_is( Persistent& object )
{
	const int SIZE = 100;
	char mes[SIZE];
	cout << "It is a " << object.portrait(mes,SIZE) << "\n";
}

//	Remember allocate template of "Float" and "Int"
//	But do not delete them

void main1()
{
	Object_maker::record("Float", new Float);
	Object_maker::record("Int"  , new Int  );

	Persistent* a = Object_maker::make_a( "Float", "3.12" );
	Persistent* b = Object_maker::make_a( "Int" , "3" );

	what_is( *a ); what_is( *b );

	cout << "Int   = " << *((Int*)b) << "\n";
	cout << "Float = " << *((Float*)a) << "\n";

	Float aa = *( (Float*) a );
	cout << "Float = " << aa << "\n";

}

// From the stream in
//	extract the class name and the objects value as C_strs
//	Stored as  'class name : data values newline'

Boolean extract( ifstream& in, String& name, String& value )
{
	enum State { NAME, VALUE };
	State  component = NAME;

	name = ""; value = "";
	char c; in >> c;
	if ( c == '[' )
		in >> c;
	else
		return FALSE;			// Corrupt file

	if ( in.eof() ) return FALSE;		// premature EOF

	while( c != ']' )
	{
		if ( in.eof() ) return FALSE;	// premature EOF
		if ( c == ':' )
			component = VALUE;
		else {
			if ( component == NAME )
				name = name + c;
			else
				value = value + c;
		}
		in >> c;
	}
	return TRUE;
}

//	Remember allocate template of "Float" and "Int"
//	But do not delete them

void main3()
{
	const int SIZE = 100, NUMBER = 5;
	char desc[SIZE];
	int i;

	Object_maker::record("Float", new Float);
	Object_maker::record("Int"  , new Int  );

	Persistent * data_items[NUMBER];
	data_items[0] = new Float(3.14);
	data_items[1] = new Int(1);
	data_items[2] = new Float(123.5);
	data_items[3] = new Float(0.5);
	data_items[4] = new Int(999);

	ofstream o_base;
	o_base.open("data.txt");

	for ( i = 0; i < NUMBER; i++ )
		o_base << data_items[i]->portrait(desc,SIZE) << "\n";

	o_base.close();

	Persistent * items[NUMBER];
	ifstream i_base;
	i_base.open("data.txt");

	int last=0;
	String class_name,object_value;

	while ( !i_base.eof() )
	{
		if ( extract( i_base, class_name, object_value ) )
			items[last++] =
				Object_maker::make_a(
					class_name.c_string(),
					object_value.c_string()
				);
	}
	i_base.close();

	for (i = 0; i < last; i++ )
	{
		cout << "Item " << i << " is a ";
		cout << items[i]->portrait(desc,SIZE) << "\n";
	}

	for (i=0; i<last; i++ )
	{
		delete items[i];
	}

}

void main2()
{
	Float a,b;
	a = 3.14; b = 2;
	cout << "Answer = " << a*b << "\n";
}

// P_Account a class who's instancies can be made persistant

class P_Account : public Account, public Persistent
{
public:
	P_Account();			// Constructor
	P_Account( C_str );		// Construct with state
	C_w_str portrait( C_w_str, const int) const; // Description
	Persistent* new_object( C_str );	// Replicate with state
};

P_Account::P_Account() : Account()
{
	// Constructor for P_Account calls Accounts constructor
}

P_Account::P_Account( C_str str )
{
	deposit( (float) atof( str ) );	// Create with specific value
}

C_w_str P_Account::portrait( C_w_str mes, const int length ) const
{
	ostrstream text(mes,length);	// Write object name and value
	text << "[P_Account: " << account_balance() << "]" << '\0';
	return mes;
}

Persistent* P_Account::new_object( C_str str )
{
	return new P_Account(str);	// create new object
}

void main4()
{
	const int SIZE_MES = 120;
	char message[SIZE_MES];

	Object_maker::record("P_Account", new P_Account);

	P_Account *mas = new P_Account;
	cout << mas->portrait(message,SIZE_MES) << "\n";

	Persistent* an_account = Object_maker::make_a("P_Account", "200");
	cout << an_account->portrait(message,SIZE_MES) << "\n";

        delete mas; delete an_account;
}


class P_S_Account : public S_Account, public Persistent
{
public:
	P_S_Account();				// Constructor
	P_S_Account( C_str );			// Construct with state
	C_w_str portrait( C_w_str, const int) const; // Description
	Persistent* new_object( C_str );	// Replicate with state
};

P_S_Account::P_S_Account() : S_Account()
{
	// Constructor for P_S_Account calls Accounts constructor
}

P_S_Account::P_S_Account( C_str str )
{
	deposit( atof( str ) );	// Create with specific value
}

C_w_str P_S_Account::portrait( C_w_str mes, const int length ) const
{
	ostrstream text(mes,length);	// Write object name and value
	text << "[P_S_Account: " << account_balance() << "]" << '\0';
	return mes;
}

Persistent* P_S_Account::new_object( C_str str )
{
	return new P_S_Account(str);	// create new object
}

// WARNING WARNING DO NOT USE THIS AS AN EXAMPLE  WARNING
// As state information in the subclasses not saved
// But shows the 'process' works

void main5()
{
	const T_Money DAILY_RATE = 0.00026116;	// 10% Annual rate
	const T_Money DAILY_RATE_R2 = 0.00028596;// 11% Annual rate
	const T_Money DAILY_RATE_R3 = 0.00031054;// 12% Annual rate

	I_Account::prelude(DAILY_RATE);

	S_Account::prelude( DAILY_RATE, DAILY_RATE_R2, DAILY_RATE_R3 );

	cout << setiosflags( ios::showpoint );
	cout << setiosflags( ios::fixed );

	const int SIZE_MES = 120;
	char message[SIZE_MES];

	Object_maker::record("P_S_Account", new P_S_Account);

	P_S_Account *mas = new P_S_Account;
	mas->deposit(100000.00);	// At start of day
	mas->calc_interest();		// For this single day
	cout << mas->portrait(message,SIZE_MES) << "\n";

	Persistent* an_account = Object_maker::make_a("P_S_Account", "200");
	cout << an_account->portrait(message,SIZE_MES) << "\n";

	delete mas;
	delete an_account;
}
