// Careful now!
//	Many compilers will put the extern "C" in in the header files
//      So this is more of a demonstration

#include <string.h>
#include <ctype.h>

extern "C" {
#include <stdio.h>
#include <stdlib.h>
}

const int MAX_LINE=150;
const int MAX_NAME=30;

enum Boolean { FALSE, TRUE };


class Source {
public:
	Source();
	~Source();
	char get_ch(void) const;
	Boolean open_file( const char text[] );
	void close_file(void) const;
private:
	char the_name[ MAX_NAME ];	// Name of source file
	FILE *the_fp;			// File pointer source file
};

Source::Source()
{
	the_fp = NULL;
}

Source::~Source()
{
	close_file();
}

Boolean Source::open_file( const char text[] )
{
	strcpy( the_name , text );
	the_fp = fopen( the_name ,"r");
	return  the_fp == NULL ? FALSE : TRUE;
}

inline char Source::get_ch(void) const
{
	return getc( the_fp );
}


void Source::close_file(void) const
{
	if ( the_fp != NULL )
	{
		fclose( the_fp );
	}
}

class Sink {
public:
	Sink();
	~Sink();
	void put_ch(const char c) const;
	Boolean open_file( const char text[] );
	void close_file(void) const;
private:
	char the_name[ MAX_NAME ];	// Name of sink file name
	FILE *the_fp;			// Sink file pointer
};

Sink::Sink()
{
	the_fp = NULL;
}

Sink::~Sink()
{
	close_file();
}

Boolean Sink::open_file( const char text[] )
{
	close_file();
	strcpy( the_name , text );
	the_fp = fopen( the_name ,"w");
	return  the_fp == NULL ? FALSE : TRUE;
}

inline void Sink::put_ch(const char c) const
{
	if ( the_fp != NULL ) putc( c , the_fp );
}

void Sink::close_file(void) const
{
	if ( the_fp != NULL )
	{
		fclose(the_fp);
	}

}


class File_name {
public:
	File_name();
	void add_ch( const char c);
	const char* deliver_file_name(void) const;
private:
	char the_text[ MAX_NAME ];	// File name as characters
	int the_position;		// extracting from position
	int the_no_characters;		// total characters in name
};

File_name::File_name()
{
	the_no_characters = 0;
	the_position = 0;
}

inline void File_name::add_ch( const char c )
{
	the_text[the_no_characters++] = c;
}

inline const char* File_name::deliver_file_name(void) const
{
	return &the_text[0];
}



class Line {
public:
	Line();
	void fill_from( Source& s );
	void empty_to( Sink& s ) const;
	Boolean is_a_command(void) const;
	Boolean at_eof(void) const;
	File_name extract_file_name(void) const;
private:
	char the_text[MAX_LINE];	// Characters in line
	int  the_no_characters;		// Number of characters in line
	char last;			// last character extracted
};

Line::Line()
{
	the_no_characters = 0;
}

void Line::fill_from( Source& s )
{
	char c; int i=0;
	while ( (c=s.get_ch()) != '\n' && c != EOF )
	{
		the_text[i++] = c;
	}
	the_text[i] = '\0';
	last = c; the_no_characters = i;
}

void Line::empty_to( Sink& s ) const
{
	int i;
	for(i=1; i<the_no_characters;i++)
		s.put_ch( the_text[i] );
	s.put_ch('\n');
}

Boolean Line::is_a_command(void) const
{
	if (the_text[0]=='@' && the_text[1]=='@' && the_no_characters>=2)
		return TRUE;
	else
		return FALSE;
}

inline Boolean Line::at_eof(void) const
{
	return (last == EOF ? TRUE : FALSE );
}

File_name Line::extract_file_name(void) const
{
	File_name fn;
	for(int i=2; the_text[i] != '\0'; i++)
	{
		char c = the_text[i];
		if ( isupper( c ) )		// If upper case convert
			c = tolower( c );       // to lower case
		if ( isalnum( c ) || c == '.' || c == '_' || 
				     c == '+' || c == '-' )
			fn.add_ch( c );		// Only alpnum . _ + -
	}
	fn.add_ch('\0');			// End string terminator
	return fn;
}

void main(int argc, char *argv[])
{
	Source  input;
	Sink    output;
	Line    current_line;

	if ( argc != 2 )
	{
		fprintf(stderr,"Usage: extract file\n");
		exit(-1);
	}

	if ( input.open_file(argv[1]) != TRUE )
	{
		fprintf(stderr,"Cannot open %s\n" , argv[1] );
		exit(-2);
	}

	current_line.fill_from( input );
	while ( current_line.at_eof() == FALSE )
	{
		if ( current_line.is_a_command() == TRUE )
		{
			File_name fn;
			fn = current_line.extract_file_name();
			if (output.open_file(fn.deliver_file_name())==FALSE)
			{
				fprintf(stderr,"Cannot create %s\n",
						fn.deliver_file_name() );
			} else {
				fprintf(stderr,"Extracting file %s\n",
						fn.deliver_file_name() );
			}
		} else {
			current_line.empty_to( output );
		}
		current_line.fill_from( input );
	}
	output.close_file();
	input.close_file();
}
