#include <iostream.h>

const int MAXROW = 9;		// Maximum Rows
const int MAXCOLUMN = 9;	// Maximum Columns

const int STONES_IN_A_WIN_LINE = 4;
const int NO_OF_PLAYERS=2;
const int DEF_ROWS = 6;		// Default number of rows
const int DEF_COLUMNS = 7;	// Default number of columns

enum Boolean { FALSE,TRUE };
enum BoardState { DRAW, WIN, PLAY };

const char INVISIBLE = ' ';
const int SELECT_REPRESENTATION = 1;

class Counter {
public:
	static void prelude(void);
	Counter();
	Counter(const char);
	Counter(const int);
	char colour(void) const;// Returns the colour of a counter
	void display(void) const;// Display the counter
private:
	static int the_counter_no;// Which colour to allocate next
	char the_colour;	// The counters colour
};


int Counter::the_counter_no;// Declaration of storage for static member

void Counter::prelude(void)
{
	the_counter_no = 0;
}

inline Counter::Counter()
{
	the_colour = INVISIBLE;
}

Counter::Counter(const int)
{
	the_colour = "xo%*@$&+~"[the_counter_no++];
}

inline Counter::Counter(const char representation)
{
	the_colour = representation;
}

inline char Counter::colour(void) const
{
	return the_colour;
}

inline void Counter::display(void) const
{
	cout << the_colour;
}

class Player {
public:
	Player();
	int get_move( const Boolean) const;// Ask the player for a move
	Counter get_counter(void) const;// Get a counter from the player
	void announce(const BoardState) const;// Announce the game state
private:
	Counter the_players_counter;
};

Player::Player()
{
	the_players_counter = Counter(SELECT_REPRESENTATION);
}

int Player::get_move( const Boolean first ) const
{
	int move;
	if ( first != TRUE )
		cout << "Error Invalid choice" << "\n";
	cout << "Move for player ";
	cout << the_players_counter.colour() << " is ";
	cin >> move;
	return move-1;
}

inline Counter Player::get_counter(void) const
{
	return the_players_counter;
}

void Player::announce(const BoardState what) const
{
	cout << "Player " << the_players_counter.colour();
	switch ( what )
	{
		case WIN :
			cout << " wins" << "\n";
			break;
		case DRAW :
			cout << " has filled the board" << "\n";
			break;
	}
}

class Cell {
public:
	Cell();
	void clear(void);	// Clear the cell
	void drop(const Counter c);// Drop a counter into the cell
	char holds(void) const;	// Return the colour of the counter
	void display(void) const;// Display the contents of the cell
private:
	Counter the_counter;	// The counter in this cell
};

Cell::Cell()
{
	the_counter = Counter(INVISIBLE);
}

inline void Cell::clear(void)
{
	the_counter = Counter(INVISIBLE);
}

inline void Cell::drop(const Counter c)
{
	the_counter = c;
}

inline char Cell::holds(void) const
{
	return the_counter.colour();
}

inline void Cell::display(void) const
{
	the_counter.display();
}

class Board {
public:
	Board(const int rows=DEF_ROWS, const int columns=DEF_COLUMNS);
	void reset(const int rows=DEF_ROWS, const int columns=DEF_COLUMNS);
	void drop_in_column( const int , const Counter);
	Boolean move_ok_for_column( const int ) const;
	void display(void) const;
	BoardState situation(void) const;
protected:
	int check_for_a_win( const int, const int,
			     const int, const char) const;
	void add_counter_to_board(const int, const Counter );
	Boolean is_there_a_win(void) const;
private:
	Cell the_grid[MAXROW][MAXCOLUMN];// Board played on
	int the_height[MAXCOLUMN];	// Current height of counters col
	int the_row_size;
	int the_column_size;		// Size of playing area
	int the_last_col;
	int the_last_row;		// Last counter placed
	int the_no_empty_cells;		// No. of cells still empty
};

Board::Board(const int rows, const int columns)
{
	reset(rows,columns);
}

void Board::reset(const int rows, const int columns)
{
	the_row_size = rows; the_column_size=columns;
	for (int i=0; i<the_column_size; i++ )
	{
		for (int j=0; j<the_row_size; j++ )
		{
			the_grid[j][i].drop(Counter(INVISIBLE));
		}
		the_height[i] = 0;
	}
	the_last_row = the_last_col = 0;
	the_no_empty_cells = rows*columns;
}

void Board::drop_in_column( const int column , const Counter c)
{
	the_last_col = column; the_last_row = the_height[column];
	add_counter_to_board( column , c );
}

Boolean Board::move_ok_for_column(const int column) const
{
	if ( column>=0 && column<the_column_size  )
	{
		if ( the_height[column] < the_row_size )
	{
			return TRUE;
		}
	}
	return FALSE;
}

void Board::add_counter_to_board(const int column,
		const Counter players_counter)
{
	the_grid[ the_height[column] ][column].drop(players_counter);
	the_height[column]++;
	the_no_empty_cells--;
}

BoardState Board::situation(void) const
{
	if ( is_there_a_win()==TRUE )
	{
		return WIN;
	}
	if ( the_no_empty_cells == 0 )
	{
		return DRAW;
	}
	return PLAY;
}

Boolean Board::is_there_a_win(void) const
{
	char stone = the_grid[the_last_row][the_last_col].holds();
	for( int dir=1; dir<=4; dir++)
	{
		int stones_in_a_line =
		 check_for_a_win(dir,the_last_row,the_last_col,stone)+
		 check_for_a_win(dir+4,the_last_row,the_last_col,stone)-1;
		if ( stones_in_a_line >= STONES_IN_A_WIN_LINE )
		{
			return TRUE;
		}
	}
	return FALSE;
}

int Board::check_for_a_win( const int dir,
	const int x_cord, const int y_cord, const char counter) const
{
	int x = x_cord; int y=y_cord;
	if ( ( x >= 0 && x < the_row_size ) &&
	     ( y >= 0 && y < the_column_size ) &&
	     ( the_grid[x][y].holds() == counter ) )
	{
		switch ( dir )
		{
			case 1:      y++; break;
			case 2: x++; y++; break;
			case 3: x++;      break;
			case 4: x++; y--; break;
			case 5:      y--; break;
			case 6: x--; y--; break;
			case 7: x--;      break;
			case 8: x--; y++; break;
			default: cerr << "internal error #1" << "\n";
		}
		return 1 + check_for_a_win( dir , x , y , counter );
	} else {
		return 0;
	}
}

void Board::display(void) const
{
	cout << "  ";
	for (int j=1; j<=the_column_size; j++)
		cout << j << "   ";
	cout << "\n";

	for (int i=the_row_size-1; i>=0; i-- )
	{
		cout << "| ";
		for (int j=0; j<the_column_size; j++ )
		{
			the_grid[i][j].display();
			cout << " | ";
		}
		cout << "\n";
	}
	for (int k=the_row_size; k>=0; k--) cout << "----";
	cout << "-" << "\n" << "\n";
}

void play();
void again();

void main()
{
	Counter::prelude();
	play();

	Counter::prelude();
	again();
}

void play()
{
	Board c_4;			// The playing board
	Player contestant[2];		// No of players
	BoardState state = PLAY;	// Contestants can Play
	int no = 0;			// First contestant

	c_4.display();			// Display the board
	while ( state == PLAY )		// While state is playable
	{
		int move = contestant[no].get_move(TRUE);
		while ( c_4.move_ok_for_column(move) == FALSE )
		{
			move = contestant[no].get_move(FALSE);
		}
		c_4.drop_in_column( move, contestant[no].get_counter() );

		c_4.display();
		state = c_4.situation();
		if ( state == PLAY )
			no = ( no == 0 ? 1 : 0 );	// Next Contestant
		else
			contestant[no].announce(state);	// The result
	}
}

void again()
{
	Board c_4;			// The playing board
	Player contestant[NO_OF_PLAYERS];// No of players
	BoardState state = PLAY;	// Contestants can Play
	int no = 0;			// First contestant

	c_4.display();			// Display the board
	while ( state == PLAY )		// While state is playable
	{
		int move = contestant[no].get_move(TRUE);
		while ( c_4.move_ok_for_column(move) == FALSE )
		{
			move = contestant[no].get_move(FALSE);
		}
		c_4.drop_in_column( move, contestant[no].get_counter() );

		c_4.display();
		state = c_4.situation();
		if ( state == PLAY )
			no = (no+1)%NO_OF_PLAYERS;// Next Contestant
		else
			contestant[no].announce(state);	// The result
	}
}
