-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


package body Class_histogram is

  procedure reset(the:in out Histogram) is
  begin
    the.number_of := ( others => 0 );  -- Reset counts to 0
  end reset;

  procedure add_to(the:in out Histogram; a_ch:in Character) is
    ch : Character;
  begin
    ch := a_ch;                        -- As write to ch
    if is_lower(ch) then               -- Convert to upper case
      ch := to_upper( ch );
    end if;
    if is_upper( ch ) then             -- so record
      declare
        c : Alphabet_index := Alphabet_index(ch);
      begin
        the.number_of(c) := the.number_of(c) + 1;
      end;
    end if;
  end add_to;

  procedure put(the:in Histogram; 
                height:in Positive:=DEF_HEIGHT) is
    frequency    : Alphabet_array;        -- Copy to process
    max_height   : Natural := 0;          -- Observed max
  begin
    frequency := the.number_of;           -- Copy data (Array)
    for ch in Alphabet_range loop         -- Find max frequency
      if frequency(ch) > max_height then
        max_height:= frequency(ch);
      end if;
    end loop;

    if max_height > 0 then
      for ch in Alphabet_range loop      -- Scale to max height
        frequency(ch):=(frequency(ch)*height)/(max_height);
      end loop;
    end if;

    for row in reverse 1 .. height loop  -- Each line
      put( "  | " );                     -- start of line
      for ch in Alphabet_range loop
        if frequency(ch) >= row then
          put('*');                      -- bar of hist >= col
        else
          put(' ');                      -- bar of hist <  col
        end if;
      end loop;
      put(" | "); new_line;              -- end of line
    end loop;
    put("  +----------------------------+"); new_line;
    put("    ABCDEFGHIJKLMNOPQRSTUVWXYZ " ); new_line;
    put("  *  = (approx) ");
    put( Float(max_height) / Float(height), aft=>2, exp=>0 );
    put(" characters "); new_line;
  end put;
end Class_histogram;
