-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Simple_io; use Simple_io;
package body Class_rational is


  function "+" (f:in Rational; s:in Rational) return Rational is
    res : Rational;
  begin
    res.below := f.below * s.below;
    res.above := f.above * s.below + s.above * f.below;
    return simplify(res);
  end "+";

  function "-" (f:in Rational; s:in Rational) return Rational is
    res : Rational;
  begin
    res.below := f.below * s.below;
    res.above := f.above * s.below - s.above * f.below;
    return simplify(res);
  end "-";

  function "*" (f:in Rational; s:in Rational) return Rational is
    res : Rational;
  begin
    res.above := f.above * s.above;
    res.below := f.below * s.below;
    return simplify(res);
  end "*";

  function "/" (f:in Rational; s:in Rational) return Rational is
    res : Rational;
  begin
    res.above := f.above * s.below;
    res.below := f.below * s.above;
    return simplify(res);
  end "/";

  function rat_const( f:in Integer; s:in Integer:=1 ) return Rational is
  begin
    if f = 0 then
      return Rational'(0,1);
    else
      return simplify( sign( Rational'( f, s ) ) );
    end if;
  end rat_const;

  procedure put( the:in Rational ) is
    above : Integer := the.above;
    below : Integer := the.below;
  begin
    if above = 0 then                           -- Rational 0
      put( "0" );
    else
      if above < 0 then
        put("-"); above := -above;              --  make +ve
      end if;
      if above >= below then                    -- Whole number
        put( above/below, width=>1 ); put(" ");
        above := above rem below;               -- Fraction
      end if;
      if above /= 0 then
        put( above, width=>1 ); put( "/" );
        put( below, width=>1 );                 -- Fraction
      end if;
    end if;
  end put;

  function sign( the:in Rational ) return Rational is
  begin
    if the.below >= 0 then            --   -a/b or a/b
      return the;
    else                              --   a/-b or -a/-b
      return Rational'( -the.above, -the.below );
    end if;
  end sign;

  function simplify( the:in Rational ) return Rational is
    res: Rational := the;
    d  : Positive;                    -- Divisor to reduce with
  begin
    if res.below = 0 then             -- Invalid treat as 0
      res.above := 0; res.below := 1;
    end if;
    d := 2;                           -- Divide by 2, 3, 4 ...
    while d < res.below loop
      while res.below rem d = 0 and then res.above rem d = 0 loop
        res.above := res.above / d;
        res.below := res.below / d;
      end loop;
      d := d + 1;
    end loop;
    return res;
  end simplify;

end Class_rational;
