--[class_stack.ads] Specification
WITH Text_io; USE Text_io;

GENERIC
  TYPE Stack_element IS PRIVATE;        --

PACKAGE Class_stack IS
  TYPE Stack IS LIMITED PRIVATE;        -- NO copying
  Stack_error : EXCEPTION;

  PROCEDURE push( the:IN OUT Stack; item:IN Stack_element );
  PROCEDURE pop(the:IN OUT Stack; item :OUT Stack_element );
  PROCEDURE reset( the:IN OUT Stack );
PRIVATE

  TYPE Node;                            -- Mutually recursive def
  TYPE P_Node IS ACCESS Node;           -- Pointer to a Node
  PRAGMA Controlled( P_node );          -- We do deallocation

  TYPE Node IS RECORD                   -- Node holds the data
    item   : Stack_element;             -- The stored item
    p_next : P_Node;                    -- Next in list
  END RECORD;

  TYPE Stack IS RECORD
    p_head : P_Node := NULL;            -- First node
  END RECORD;
END Class_stack;

--[class_stack.adb] Implementation

WITH unchecked_deallocation;
PRAGMA ELABORATE_ALL( unchecked_deallocation );
PACKAGE BODY Class_stack IS

  PROCEDURE dispose IS
    NEW unchecked_deallocation( Node, P_Node );

  PROCEDURE push( the:IN OUT Stack; item:IN Stack_element ) IS
    tmp : P_Node;                       -- Allocated node
  BEGIN
    tmp := NEW Node'(item=>item, p_next=>the.p_head);
    the.p_head := tmp;
  END push;

  PROCEDURE pop( the:IN OUT Stack; item :OUT Stack_element ) IS
    tmp : P_Node;                        -- Free node
  BEGIN
    IF the.p_head /= NULL THEN           -- if item then
      tmp := the.p_head;                 -- isolate top node
      item := the.p_head.item;           -- extract item stored
      the.p_head := the.p_head.p_next;   -- Relink
      dispose( tmp );                    -- return storage
    ELSE
      RAISE Stack_error;                 -- Failure
    END IF;
  END pop;

  PROCEDURE reset( the:IN OUT Stack ) IS
    tmp : Stack_element;
  BEGIN
    WHILE the.p_head /= NULL LOOP      -- Re-initialize stack
      pop( the, tmp );
    END LOOP;
  END reset;

END Class_stack;

--[class_stack_int.ADS] Specification
WITH Class_stack;
PRAGMA ELABORATE_ALL( Class_stack );
PACKAGE Class_stack_int IS NEW Class_stack(Integer);

--[main.adb] Procedure
WITH Simple_io, Class_stack_int;
USE  Simple_io, Class_stack_int;
PROCEDURE main IS
  number_stack : Stack;              -- Stack of numbers
  action       : Character;          -- Action
  number       : Integer;            -- Number processed
BEGIN
  reset( number_stack );             -- Reset stack to empty
  WHILE NOT end_of_file LOOP
    WHILE NOT end_of_line LOOP
      BEGIN
        get( action );
        CASE action IS               -- Process action
          WHEN '+' =>
            get( number ); push(number_stack,number);
            put("push number = "); put(number); new_line;
          WHEN '-' =>
            pop(number_stack,number);
            put("Pop number  = "); put(number); new_line;
          WHEN OTHERS =>
            put("Invalid action"); new_line;
        END CASE;
      EXCEPTION
        WHEN Stack_error =>
          put("Stack_error"); new_line;
        WHEN Data_error  =>
          put("Not a number"); new_line;
        WHEN End_error   =>
          put("Unexpected end of file"); new_line; EXIT;
      END;
    END LOOP;
    skip_Line;
  END LOOP;

  reset( number_stack );
END main;
