-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


--[class_stack.adb] Implementation

WITH unchecked_deallocation;
PRAGMA ELABORATE_ALL( unchecked_deallocation );
PACKAGE BODY Class_stack IS

  PROCEDURE dispose IS
    NEW unchecked_deallocation( Node, P_Node );

  PROCEDURE push( the:IN OUT Stack; item:IN Stack_element ) IS
    tmp : P_Node;                       -- Allocated node
  BEGIN
    tmp := NEW Node'(item=>item, p_next=>the.p_head);
    the.p_head := tmp;
  END push;

  PROCEDURE pop( the:IN OUT Stack; item :OUT Stack_element ) IS
    tmp : P_Node;                        -- Free node
  BEGIN
    IF the.p_head /= NULL THEN           -- if item then
      tmp := the.p_head;                 -- isolate top node
      item := the.p_head.item;           -- extract item stored
      the.p_head := the.p_head.p_next;   -- Relink
      dispose( tmp );                    -- return storage
    ELSE
      RAISE Stack_error;                 -- Failure
    END IF;
  END pop;

  PROCEDURE reset( the:IN OUT Stack ) IS
    tmp : Stack_element;
  BEGIN
    WHILE the.p_head /= NULL LOOP      -- Re-initialize stack
      pop( the, tmp );
    END LOOP;
  END reset;

END Class_stack;
