-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


--with Simple_io; use Simple_io;
package body Class_list.Iterator is

  procedure initialize( the:in out List_iter ) is
  begin
    the.cur_node       := null;   -- Iterator not setup
    the.cur_list_first := null;
    the.cur_list_last  := null;
  end initialize;

  procedure finalize( the:in out List_iter ) is
  begin
    null;
  end finalize;

  procedure first( the:in out List_iter; l:in out List ) is
  begin
    the.cur_node      := l.first_node;       -- Set to first
    the.cur_list_first:= l.first_node'Unchecked_Access;
    the.cur_list_last := l.last_node'Unchecked_Access;
  end first;

  procedure last( the:in out List_iter; l:in out List ) is
  begin
    the.cur_node      := l.last_node;        -- Set to last
    the.cur_list_first:= l.first_node'Unchecked_Access;
    the.cur_list_last := l.last_node'Unchecked_Access;
  end last;

  function deliver( the:in List_iter ) return T is
  begin
    return the.cur_node.item;  -- The current item
  end deliver;

  procedure insert( the:in out List_iter; data:in T ) is
    tmp   : P_Node;
    cur   : P_Node   := the.cur_node;   -- Current element
    first : P_P_Node := the.cur_list_first;
    last  : P_P_Node := the.cur_list_last;
  begin
    if cur = null then            -- Empty or last item
      if first.all = null then    --  Empty list
        tmp := new Node'( null, data, null );
        first.all := tmp;
        last.all  := tmp;
        the.cur_node := tmp;
      else                        --  Last
        tmp := new Node'( last.all, data, null );
        last.all.next := tmp;
        last.all      := tmp;
        the.cur_node := tmp;
       end if;
    else
      tmp := new Node'( cur.prev, data, cur );
      if cur.prev = null then      -- First item
        first.all := tmp;
      else
        cur.prev.next := tmp;
      end if;
      cur.prev := tmp;
    end if;
  end insert;

  procedure delete( the:in out List_iter) is
    cur   : P_Node   := the.cur_node;   -- Current element
    first : P_P_Node := the.cur_list_first;
    last  : P_P_Node := the.cur_list_last;
  begin
    if cur /= null then             -- Something to delete
      if cur.prev /= null then      -- Fix forward pointer;
        cur.prev.next := cur.next;  --  Not first in chain
      else
        first.all := cur.next;      --  First in chain
        if first.all = null then
          last.all := null;         --   Empty list
        end if;
      end if;
      if cur.next /= null then      -- Fix backward pointer;
        cur.next.prev := cur.prev;  --  Not last in chain
      else
        last.all := cur.prev;       --  Last in chain
        if last.all = null then
          first.all := null;        --   Empty list
        end if;
      end if;
      if cur.next /= null then      -- Fix current pointer
        the.cur_node := cur.next;   --  next
      elsif cur.prev /= null then
        the.cur_node := cur.prev;   --  previous
      else
        the.cur_node := null;       --  none empty list
      end if;
      dispose_node( cur );          -- Release storage
    end if;
  end delete;

  function  is_end( the:in List_iter ) return Boolean is
  begin
    return the.cur_node = null;               -- True if end
  end is_end;

  procedure next( the:in out List_iter ) is
  begin
    if the.cur_node /= null then               --
      the.cur_node  := the.cur_node.next;      -- Next
    end if;
  end next;

  procedure prev( the:in out List_iter ) is
  begin
    if the.cur_node /= null then               --
      the.cur_node  := the.cur_node.prev;      -- Previous
    end if;
  end prev;

end Class_list.Iterator;
