-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Simple_io; use Simple_io;
package body Pack_procedures is

procedure ex1 is
  numbers    : List;            -- List of Natural numbers
  numbers_it : List_iter;       -- Iterator for list
begin
  first( numbers_it, numbers );
  insert( numbers_it, 50 );
  insert( numbers_it, 5 );
  insert( numbers_it, 40 );
  last( numbers_it, numbers );
  next( numbers_it );
  insert( numbers_it, 100 );
  prev( numbers_it );
  prev( numbers_it );
  insert( numbers_it, 30 );
  first( numbers_it, numbers );
  while not is_end( numbers_it ) loop
    put( deliver( numbers_it ) );
    next( numbers_it );
  end loop;
  new_line;
end ex1;

procedure ex2 is
  numbers    : List;            -- List of Natural numbers
  numbers_it : List_iter;       -- Iterator for list

procedure insert( pos:in Integer; num:in Integer ) is
  count : Integer := pos;
begin
  first( numbers_it, numbers );
  while count >= 1 loop
    next( numbers_it ); count := count-1;
  end loop;
  insert( numbers_it, num );
end insert;

procedure delete( pos:in Integer ) is
  count : Integer := pos;
begin
  first( numbers_it, numbers );
  while count >= 1 loop
    next( numbers_it ); count := count-1;
  end loop;
  delete( numbers_it );
end delete;

procedure print is
begin
  first( numbers_it, numbers );
  while not is_end(numbers_it) loop
    put( deliver(numbers_it), width=>3 );
    next( numbers_it );
  end loop;
  new_line;
end print;

begin
  insert( 0, 50 ); print;
  put(" 50 <Should be>"); new_line;
  insert( 0, 30 ); print;
  put(" 30 50 <Should be>"); new_line;
  insert( 1, 40 ); print;
  put(" 30 40 50 <Should be>"); new_line;
  insert( 2, 45 ); print;
  put(" 30 40 45 50 <Should be>"); new_line;
  insert( 2, 42 ); print;
  put(" 30 40 42 45 50 <Should be>"); new_line;
  insert( 5, 99 ); print;
  put(" 30 40 42 45 50 99<Should be>"); new_line;

  delete( 5 ); print;
  put(" 30 40 42 45 50 <Should be>"); new_line;
  delete( 0 ); print;
  put(" 40 42 45 50 <Should be>"); new_line;
  delete( 1 ); print;
  put(" 40 45 50 <Should be>"); new_line;
  delete( 2 ); print;
  put(" 40 45 <Should be>"); new_line;
  delete( 1 ); print;
  put(" 40 <Should be>"); new_line;
  delete( 0 ); print;
  put(" <Should be>"); new_line;
end ex2;

procedure ex3 is
  numbers    : List;            -- List of Natural numbers
  numbers_it : List_iter;       -- Iterator for list
begin
  first( numbers_it, numbers );
  insert( numbers_it, 100 );
  insert( numbers_it, 50 );

  first( numbers_it, numbers );
  while not is_end(numbers_it) loop
    put( deliver(numbers_it) ); next( numbers_it );
  end loop;
  new_line;
end ex3;

procedure ex4 is
  numbers    : List;               -- List of Natural numbers
  numbers_it : List_iter;          -- Iterator for list
begin
  first( numbers_it, numbers );    -- Initialize
  for number in 1 .. 10 loop
    insert( numbers_it, number );  -- Insert before
    next( numbers_it );            -- Next item
  end loop;

  first(numbers_it,numbers);                -- Set to start
  while not is_end( numbers_it ) loop       -- Not end of list
    put( deliver(numbers_it) ); put(" ");   --  Print
    next( numbers_it );                     -- Next item
  end loop;
  new_line;
end ex4;

procedure ex5 is
  original: List;                      -- List of numbers
  copy    : List;                      -- List of numbers
  num_it  : List_iter;                 -- Iterator
begin
  for number in 1 .. 10 loop
    last( num_it, original );          -- Set iterator last
    next( num_it );                    -- So can insert
    insert( num_it, number );          --   after last item
  end loop;

  put("copy := original "); new_line;
  copy := original;

  put("copy = original ");
  if copy = original then put("True"); else put("False"); end if;
  new_line;

  put("Append 99 to copy "); new_line;
  last( num_it, copy ); next( num_it );
  insert( num_it, 99 );

  put("copy = original ");
  if copy = original then put("True"); else put("False"); end if;
  new_line;

  first(num_it,original);               -- Set to start
  while not is_end( num_it ) loop       -- Not end of list
    put( deliver(num_it), width=>4 );   --  Print
    next( num_it );                     -- Next item
  end loop;

  new_line;
  first(num_it,copy);                   -- Set to start
  while not is_end( num_it ) loop       -- Not end of list
    put( deliver(num_it), width=>4 );   --  Print
    next( num_it );                     -- Next item
  end loop;
  new_line;
end ex5;

procedure ex6 is
  numbers: List;                       -- List of numbers
  num_it : List_iter;                  --  Iterator
  num,in_list: Natural;
begin
  first( num_it, numbers );

  while not end_of_file loop           -- Data available
    while not end_of_line loop
      get(num); first(num_it,numbers); -- Read number
      while not is_end( num_it ) loop  -- Scan through list
          in_list := deliver(num_it);
          exit when in_list > num;     -- Exit when larger no.
          next( num_it );              -- Next item
      end loop;
      insert( num_it, num );           -- Before larger no.
    end loop;
    skip_line;                         -- Next line
  end loop;

  put("Numbers sorted are: ");
  first(num_it,numbers);               -- Set at start
  while not is_end( num_it ) loop
    in_list := deliver( num_it );      -- Current number
    put( in_list,width=>1 ); put(" "); --  Print
    next( num_it );                    -- Next number
  end loop;
  new_line;
end ex6;


end Pack_procedures;
