-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


--with Simple_io; use  Simple_io;
package body Class_list is

  procedure initialize( the:in out List ) is
  begin
    the.first_node := null;   -- Empty list
    the.last_node  := null;   -- Empty list
  end initialize;

  procedure initialize( the:in out List; data:in T ) is
  begin
    the.first_node := new Node'(null, data, null);
    the.last_node  := the.first_node;
  end initialize;

  procedure finalize( the:in out List ) is
  begin
    if the.first_node /= null then
      release_storage( the );
      the.first_node := null;
    end if;
    --xyz( the ); -- Can not do this here DEBUG TRACE
  end finalize;

  procedure adjust( the:in out List ) is
    cur : P_Node := the.first_node;  -- Original list
    lst : P_Node := null;            -- Last created node
    prv : P_Node := null;            -- Previously created node
    fst : P_Node := null;            -- The first node

  begin
    while cur /= null loop
      lst := new Node'( prv, cur.item, null );
      if fst =  null then fst := lst; end if;
      if prv /= null then prv.next := lst; end if;
      prv := lst;
      cur := cur.next;               -- Next node
    end loop;
    the.first_node := fst;           -- Update
    the.last_node  := lst;
  end adjust;

  function "="  ( f:in List; s:in List ) return Boolean is
    f_node : P_Node := f.first_node;  -- First list
    s_node : P_Node := s.first_node;  -- Second list
  begin
    while f_node /= null and s_node /= null loop
      if f_node.item /= s_node.item then
        return FALSE;                 -- Different items
      end if;
      f_node := f_node.next; s_node := s_node.next;
    end loop;
    return f_node = s_node;           -- Both null if equal
  end "=";

  procedure release_storage( the:in out List ) is
    cur : P_Node := the.first_node; -- Pointer to curr node
    tmp : P_Node;                   -- Node to dispose
  begin
    while cur /= null loop          -- For each item in list
      tmp := cur;                   -- Item to dispose
      cur := cur.next;              -- Next node
      dispose_node( tmp );          -- Dispose of item
    end loop;
  end release_storage;

end Class_list;
