-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Ada.Text_io;
use  Ada.Text_io;
package body Pack_threads is

  protected body PT_buffer is

    entry put( ch:in Character; no_more:in Boolean )
      when no_in_queue < QUEUE_SIZE  is
    begin
        if no_more then                    -- Last
          fin := TRUE;                     -- Set flag
        else
          elements( tail ) := ch;          -- Add to queue
          tail := tail+1;                  -- Next position
          no_in_queue := no_in_queue + 1;  --
        end if;
    end;

    entry get(ch:in out Character; no_more:out Boolean)
      when no_in_queue > 0 or else fin is
    begin
      if no_in_queue > 0 then             -- Item available
        ch := elements( head );           -- Get item
        head := head+1;                   -- Next position
        no_in_queue := no_in_queue - 1;   --
        no_more := FALSE;                 -- Not end
      else
        no_more := TRUE;                  -- End of items
      end if;
    end;

  end PT_buffer;

  task body Task_read is                  -- Task implementation
    ch      : Character;
  begin
    while not end_of_file( fd_in.all ) loop
      while not end_of_line( fd_in.all ) loop
        get( fd_in.all, ch);              -- Get character
        p_buffer.put( ch, FALSE );        -- Add to buffer
        --delay 0.00001;                  -- Cause task switch
      end loop;
      skip_line( fd_in.all );             -- Next line
      p_buffer.put( CR, FALSE );          -- New line
      --delay 0.00001;                    -- Cause task switch
    end loop;
    p_buffer.put( EOT, TRUE );            -- End of characters

    accept finish;
  exception
    when Tasking_error =>
      put("Exception in Task read"); new_line;
  end Task_read;

  task body Task_write is                 -- Task implementation
    last     : Boolean := FALSE;          -- No more data
    ch       : Character;                 -- Character read
  begin
    loop
      p_buffer.get( ch, last );           -- From buffer
      exit when last;                     -- No more characters
      if ch = CR then
        new_line( fd_out.all );           -- New line
      else
        put( fd_out.all, ch );            -- Character
      end if;
      --delay 0.00001;                    -- Cause task switch
    end loop;

    accept finish;                        -- Finished
  exception
    when Tasking_error =>
      put("Exception in Task write"); new_line;
  end Task_write;
end Pack_threads;
