-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


----------------------------------------------------------------------

with Simple_io,
     Ada.Io_exceptions, Class_store, Class_file, Class_display, Pack_constants,
     Pack_ansi_display, Class_user;
use  
     Ada.Io_exceptions, Class_store, Class_file, Class_display, Pack_constants,
     Pack_ansi_display, Class_user;
procedure ed is
  MAX_OPEN  : CONSTANT := 3;           -- Maximum open files
  type Open is range 1 .. MAX_OPEN;

  c       : Open;                      -- Current screen/file
  in_store: array(Open) of Store;      -- In store copy of file
  on_disk : array(Open) of File;       -- Disk file access
  screen  : array(Open) of Display;    -- The Display
  person  : User;                      -- The editors user


function yes( str:in String ) return Boolean is
begin
  return ( str'Length = 1  and then
           (str = "y" or  else str = "Y") ) or else
         ( str'Length = 3  and then
           (str = "yes" or else str = "YES") );
end yes;

function no( str:in String ) return Boolean is
begin
  return ( str'Length = 1  and then
          (str = "n" or  else str = "N") ) or else
         ( str'Length = 2  and then
          (str = "no" or else str = "NO") );
end no;

procedure open_file( the:in Open ) is      -- Read file into buffer
  fail : Boolean := FALSE;               -- Result of read
begin
  loop                                   -- Repeat until read
    begin
      if is_active( on_disk(c) ) and then -- Deleting current
         not yes(dialog( person,
                 "Delete this buffer [y/n] : ")) then
        exit;                            -- No
      end if;
      clear( in_store(c) );              -- Clear store
      register(on_disk(c),
               dialog(person, "File name: "));
      read( on_disk(c), in_store(c) );   -- Read file into store
      init( screen(c), in_store(c) );    -- Initialize screen
      refresh( screen(c) );              -- Display
      set_not_active( on_disk(c) );      -- File not active
      exit;
    exception
      when Name_error => fail := TRUE;   -- Could not read
      when others     => fail := TRUE;   -- Anything else
    end;
  end loop;
  if fail then                           -- If failed to read
    clear( in_store(c) );                --  clear changes
  end if;
end open_file;                           --

procedure close_file( the:in Open ) is     -- Write buffer
begin
  loop
    begin
      if not no(dialog(person, "Save file [y/n] : ")) then
        write( on_disk(c), in_store(c),
               person );                 -- Write file back
        clear( in_store(c) );            -- clear data
        init( screen(c), in_store(c) );  -- Set to empty
        position(screen(c), in_store(c));-- set to start position
        refresh( screen(c) );            -- Blank screen
        set_not_active( on_disk(c) );    -- Now non active
      end if;
      exit;
    exception
      when Name_error =>
        register( on_disk(c), " " );      -- Could not write
      when others     =>
        register( on_disk(c), " " );      -- Could not write
    end;
  end loop;
end close_file;

procedure commands is
begin
  Simple_io.new_line;
  Simple_io.put(" Left      ^L         Right       ^R"); Simple_io.new_line;
  Simple_io.put(" Up        ^U         Down        ^K"); Simple_io.new_line;
  Simple_io.put(" Page Up   ^W         Page Down   ^X"); Simple_io.new_line;
  Simple_io.put(" Quit      ^E         Debug info  ^T"); Simple_io.new_line;
  Simple_io.put(" Refresh   ^Y         Del         ^H"); Simple_io.new_line;
  Simple_io.put(" Opem file ^A         Close File  ^B"); Simple_io.new_line;
  Simple_io.put(" Set file  ^F         Next Buffer ^G"); Simple_io.new_line;
end commands;


procedure process_command(action:in Character) is
begin
  case action is
    when C_OPEN   => open_file(c);       -- read file
    when C_CLOSE  => close_file(c);      -- write
    when C_SET_FN =>
      register(on_disk(c),
               dialog( person, "Set file name: "));
    when C_NEXT   =>                     -- next screen
      c := c rem MAX_OPEN + 1;
      refresh( screen(c) );
    when C_LEFT | C_RIGHT =>             -- Move   -> <-
      left_right( in_store(c), action );
      position( screen(c), in_store(c) );
    when C_UP | C_DOWN =>                -- Move   up down
      up_down( in_store(c), action, 1 );
      position( screen(c), in_store(c) );
    when C_PAGE_UP | C_PAGE_DOWN =>      -- Move   page up down
      up_down(in_store(c), action, PAGE_RACK);
      position( screen(c), in_store(c) );
    when C_DEBUG =>
      clear; debug( in_store(c) ); debug( screen(c) );
      commands;
    when C_REFRESH =>                    -- Refresh screen
      refresh( screen(c) );
    when C_DEL =>                        -- Delete Character
      del(in_store(c));
      del(screen(c), in_store(c) );
      set_active( on_disk(c) );
    when Character'Val(32) .. Character'Val(127) =>
      add(in_store(c), action);
      add(screen(c), in_store(c), action);
      set_active( on_disk(c) );
    when Ascii.cr | Ascii.lf =>          -- Ignore
      null;
    when others =>                       -- Insert ?
      add(in_store(c), '?');
      add(screen(c), in_store(c), '?');
      set_active( on_disk(c) );
  end case;
end process_command;

begin                                    -- Editor is
  for c in Open loop                     -- In each current ...
    init( screen(c), in_store(c) );      -- Initialize
  end loop;

  c := 1;                                -- Current screen #1
  refresh( screen(c) );                  -- Display cur screen
  status( screen(c), in_store(c) );      -- Display status

  loop                                   -- Main loop
    declare
      action : Character;
    begin
      action := get_command( person );   -- Editing command
      exit when action = C_QUIT;         -- Quit
      process_command( action );         -- Do action
      status( screen(c), in_store(c) );  -- Display status
    end;
  end loop;

  for c in Open loop                     -- Write changes
    if is_active( on_disk(c) ) then      -- if needed
      refresh( screen(c) );              -- display file
      close_file(c);                     -- Save file
    end if;
  end loop;

end ed;
