-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


-- begin file io.c
-- /*
--  *   The C function to turn of echoing
--  *
--  *   Works with the GNAT implementation on Linux, Win95
--  *   Note:
--  *    Uses Unix API call to turn of echoing
--  *   Compile: gcc -c io.c
--  */
-- 
-- 
-- #include <termios.h>
-- #include <unistd.h>
-- 
-- void c_no_echo()
-- {
--   static tcflag_t c_lflag;
--   static int fd = STDIN_FILENO;
--   static struct termios termios_data;
--   tcgetattr( fd, &termios_data );
--   c_lflag = termios_data.c_lflag;
--   termios_data.c_lflag = termios_data.c_lflag & (~ECHO);
--   tcsetattr( fd, TCSANOW, &termios_data );
-- }
-- end file io.c


-- ///////////////////////////////////////////////////////////////////////////
-- 
-- Alternative body for Raw_io
--   All the I/O is performed by code written in C
--   Implemented in C code that following this Ada body
-- 

-- with Interfaces.C;
-- use  Interfaces.C;
-- package body raw_io is
-- 
-- procedure get_immediate( ch:out Character ) is
--    function c_get_char return Char;
--    pragma import (C, c_get_char, "c_get_char");
-- begin
--    ch := to_ada( c_get_char );
-- end get_immediate;
-- 
-- procedure put( ch:in Character ) is
--    procedure c_put_char( ch:in Char );
--    pragma import (C, c_put_char, "c_put_char");
-- begin
--    c_put_char( to_c( ch ) );
-- end put;
-- 
-- procedure put( str:in String ) is
--    procedure c_put_str( str:in Char_array );
--    pragma import (C, c_put_str, "c_put_str");
-- begin
--    c_put_str( to_c( str, append_nul=>TRUE ) );
-- end put;
-- 
-- end raw_io;
-- 
-- C Interface for above Ada body
-- 
-- #define UNIX     1       /* Using GNAT on a UNIX based system all IO in C*/
-- #define doS      2       /* Using GNAT on a doS based system */
-- #define WIN95    3       /* Using GNAT on a Win 95/NT system */
-- #define UNIXs    WIN95   /* Using GNAT on a UNIX based system */
-- 
-- /* Notes:
--  *  ENVIRONMENT = UNIX  - Uses Unix API for all I/O
--  *              = doS   - Uses doS API (Via DJGPP C) for all I/O
--  *              = GNAT  - Uses Ada95 Input & Output procedures +
--  *                             Unix API call to turn of echoing of input
--  *              = UNIXs - Uses Ada95 Input & Output procedures +
--  *                             Unix API call to turn of echoing of input
--  */
-- 
-- #define ENVIRONMENT	WIN95    /* Environment for program */
-- 
-- 
-- #define ESC	'\033'
-- 
-- 
-- #if ENVIRONMENT == doS
-- # include <pc.h>
-- # include <keys.h>
-- #endif
-- #include <stdio.h>
-- 
-- typedef enum { false, true } bool;
-- 
-- char c_get_char();
-- void c_put_char( char ch );
-- void c_put_str( char *str );
-- 
-- 
-- #if ENVIRONMENT == doS
-- /*
--  * Make function keys and arrow keys return two characters
--  * E.G. Right arrow returns (char) 0, 'M'
--  *      Left  arrow         (char) 0, 'K'
--  */
-- 
-- char c_get_char()
-- {
--   int c;
--   static char the_ch;                 /* Remembered character */
--   static bool prev_char = false;      /* There is remembered ch */
--   if ( prev_char ) {
--     prev_char = false; return the_ch;
--   }
--   c = getkey();                        /* Get char no echo */
--   if ( c & 0x100 ) {                   /* Function / Arrow key */
--     prev_char = true; 
--     the_ch = (char) ( c & 0xFF );
--     return (char) 0;                   /* Marker */
--   }
--   return (char) (c & 0xFF);            /* Ordinary character */
-- }
-- #endif
-- 
-- #if ENVIRONMENT == UNIX
-- /*
--  * Set the terminal mode to -echo -icanon on first read
--  * reset when get ^E
--  *
--  */
-- 
-- #include <termios.h>
-- #include <unistd.h>
-- 
-- char c_get_char()
-- {
--   static bool first_time = true;
--   static tcflag_t c_lflag;
--   static int fd = STDOUT_FILENO;
--   static struct termios termios_data;
--   char c;
-- 
--   if ( first_time )
--   {
--     tcgetattr( fd, &termios_data );
--     c_lflag = termios_data.c_lflag;
--     termios_data.c_lflag = termios_data.c_lflag & ( ~(ECHO|ICANON) );
--     tcsetattr( fd, TCSANOW, &termios_data );
--     first_time = false;
--   }
--   c = getchar();
--   if ( c == '\005')
--   {
--     termios_data.c_lflag = c_lflag;
--     tcsetattr( fd, TCSANOW, &termios_data );
--   }
--   return (char) (c & 0xFF);            /* Ordinary character */
-- }
-- 
-- #endif
-- 
-- #if ENVIRONMENT == WIN95
-- 
-- /*
--  * Uses the C function c_no_echo to turn of echoing of input
--  *
--  */
-- 
-- #include <termios.h>
-- #include <unistd.h>
-- 
-- void c_no_echo()
-- {
--   static tcflag_t c_lflag;
--   static int fd = STDIN_FILENO;
--   static struct termios termios_data;
--   tcgetattr( fd, &termios_data );
--   c_lflag = termios_data.c_lflag;
--   termios_data.c_lflag = termios_data.c_lflag & (~ECHO);
--   tcsetattr( fd, TCSANOW, &termios_data );
-- }
-- 
-- #endif
-- 
-- #if ENVIRONMENT == UNIX || ENVIRONMENT == doS
-- 
-- /*
--  * C function to write characters immediately to the terminal
--  */
-- 
-- void c_put_char( char ch )
-- {
--   fputc(ch, stdout); fflush( stdout );  /* Output ch */
-- }
-- 
-- void c_put_str( char *str )
-- {
--   while (*str) fputc(*str++, stdout);  /* Output String */
--   fflush( stdout );                    /* Flush buffer */
-- }
-- 
-- #endif
-- 
-- ///////////////////////////////////////////////////////////////////////////

with Interfaces.C, Ada.Text_io;
use  Interfaces.C, Ada.Text_io;
package body Raw_io is

  procedure get_immediate( ch:out Character) is
    procedure c_no_echo;
    pragma import (C, c_no_echo, "c_no_echo");   -- Turn off echo
  begin
    if first_time then
      c_no_echo; first_time := false;
    end if;
    Ada.Text_io.get_immediate(ch);
    if Character'Pos(ch) = 10 then               -- Real Return ch
      ch := Character'Val(13);
    end if;
  end get_immediate;

  procedure put( ch:in Character ) is            -- Raw write
  begin
    Ada.Text_io.put( ch ); Ada.Text_io.flush;
  end put;

  procedure put( str:in String ) is              -- Raw write
  begin
    Ada.Text_io.put( str ); Ada.Text_io.flush;
  end put;

end Raw_io;
