-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Pack_constants, Pack_md_io, Pack_ansi_display, Simple_io;
use  Pack_constants, Pack_md_io, Pack_ansi_display;
package body Class_display is

  procedure debug( the:in Display ) is
  begin
    Simple_io.put("Display >");
    Simple_io.put("no_lines     :");
    Simple_io.put(Integer(the.no_lines),WIDTH=>2);
    Simple_io.put(" current_row :");
    Simple_io.put(Integer(the.row),WIDTH=>2);
    Simple_io.put(" current_col :");
    Simple_io.put(Integer(the.col),WIDTH=>2);
    Simple_io.put(" abs_line    :");
    Simple_io.put(Integer(the.abs_line),WIDTH=>2);
    Simple_io.new_line;
  end debug;

  procedure initialize( the:in out Display ) is
  begin
    the.row := 1;      the.col := 1;
    the.abs_line := 1; the.no_lines := 0;
--O    for i in Display_range loop
--O      initialize( the.display( i ) );    -- Individual lines
--O    end loop;
    clear(the);
  end initialize;

  procedure finalize( the:in out Display ) is
  begin
    null;
  end finalize;

  procedure clear( the:in out Display ) is
  begin
    clear;
  end clear;

  procedure init( the:in out Display; s:in Store) is
    store_row, store_col, lines : Natural;
  begin
    deliver_row_column( s, store_row, store_col );
    the.abs_line := store_row;         -- Shadowing screen from
    the.row := store_row;              --
    the.col := store_col;              -- Position on screen
    lines := no_lines(s)-store_row+1;  -- Current pos to end
    the.no_lines := Display_index(Natural'Min(lines, DisPLAY_LINES));

    for i in 1 .. the.no_lines loop
      the.display(i) := deliver_line( s, Natural(i) );
    end loop;
  end init;

  procedure refresh( the:in out Display ) is
  begin
    clear( the );
    for i in 1 .. the.no_lines loop
      cursor_position( Natural(i), 1 );
      display_line( the, i );
    end loop;
    cursor_position( the.row, the.col );
  end refresh;

  procedure display_line(the:in out Display; i:in Display_index) is
  begin
    start( the.display(i) );
    while not end_of_line( the.display(i) ) loop
      put( get_char( the.display(i) ) );
      next_ch( the.display(i) );
    end loop;
    clear_to_end_of_line;
  end display_line;

  -- Add the character to the current position in the display

  procedure add(the:in out Display; b:in Store; ch:in Character) is
    i : Display_index;
  begin
    i := Display_index( the.row );
    if i > the.no_lines then      -- Insert on empty line
      the.no_lines := i;          --  the first time
    end if;
    the.display(i):=deliver_line(b,Natural(i)+the.abs_line-1);
    cursor_position( Natural(i), 1 );
    display_line( the, i );
    the.col := deliver_current_col( the.display(i) );

    cursor_position( Natural(i), the.col );
  end add;

  -- Delete the char at the current position from the display

  procedure del( the:in out Display; s:in Store ) is
    i : Display_index;
  begin
    i := Display_index( the.row );
    the.display(i) :=
        deliver_line(s, Natural(i)+the.abs_line-1 );
    cursor_position( Natural(i), 1 );
    display_line( the, i );
    position( the, s );
  end del;

  -- Position the cursor in the correct position on the display

  procedure position( the:in out Display; s:in Store ) is
    row,column   : Natural;     -- In store
    change       : Boolean;     -- Change display
    old_abs_line : Natural;     -- Old abs line at top of screen
  begin
    deliver_row_column( s, row, column );
    change := FALSE;
    if row < the.abs_line then                     -- Rack Down
      old_abs_line := the.abs_line;
      the.abs_line := row;
      change := TRUE;
    end if;

    if row > the.abs_line + (DisPLAY_LINES-1) then  -- Rack up
      old_abs_line := the.abs_line;
      the.abs_line := row - (DisPLAY_LINES-1);
      change := TRUE;
    end if;

    if change then                            -- change display
      declare
        remaining_lines : Natural;
      begin
        remaining_lines := no_lines(s) - the.abs_line+1;
        remaining_lines := Natural'Min(remaining_lines, DisPLAY_LINES);
        the.no_lines := Display_range( remaining_lines );

        for i in 1 .. the.no_lines loop
          the.display(i) :=
              deliver_line(s, Natural(i) + the.abs_line-1 );
        end loop;

        row := row - the.abs_line + 1;
        minimal_refresh( the, old_abs_line );    -- try and do
      end;
    else
      row := row - the.abs_line + 1;
    end if;

    the.row := row; the.col := column;
    cursor_position( the.row, the.col );
  end position;

  -- Do a minimal refresh of the screen

  procedure minimal_refresh( the:in out Display;
                             old_abs_line:in Natural ) is
    diff : Natural;
  begin
    if ANSI_INS_DEL and then
       ( old_abs_line-(DisPLAY_LINES-1) <= the.abs_line and
         old_abs_line+(DisPLAY_LINES-1) >= the.abs_line )
    then
      -- Some of the lines on the screen OK
      if the.abs_line < old_abs_line then
        -- Rack display down, Insert new lines at top
        diff := old_abs_line-the.abs_line;
        cursor_position( 1, 1 );
        insert_line( diff );
        for i in 1 .. Display_index(diff) loop
          cursor_position( Natural(i), 1 );
          display_line( the, i );
        end loop;
      else
        -- Rack display up, Insert new lines at bottom of display
        diff := the.abs_line - old_abs_line;
        cursor_position(1,1); delete_line( diff );   -- Rack up
        cursor_position( DisPLAY_LINES-(diff)+1, 1);
        for i in the.no_lines-Display_index(diff)+1
                 .. the.no_lines loop
          cursor_position( Natural(i), 1 );
          display_line( the, i );
        end loop;
        for i in the.no_lines+1 .. DisPLAY_LINES loop
          cursor_position( Natural(i), 1 );      -- Clear left
          clear_to_end_of_line;                  -- on screen
        end loop;
      end if;
    else -- No lines on display valid do a total refresh
      refresh(the);
    end if;
  end minimal_refresh;

  procedure status( the:in out Display; s:in Store ) is
    row,column : Natural;
  begin
    deliver_row_column( s, row, column );
    cursor_position( LINES_ON_SCREEN, 1 ); clear_to_end_of_line;
    cursor_position( LINES_ON_SCREEN, 58 );
    put("Line "); put( row ); put("    ");
    cursor_position( LINES_ON_SCREEN, 68 );
    put("column "); put( column ); put("  ");
    cursor_position( the.row, the.col);
  end status;

  function deliver_display_size(the:in Display) return Natural is
  begin
    return DisPLAY_LINES;
  end deliver_display_size;

end Class_display;
