-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Text_io, Class_line;
use  Text_io, Class_line;
package body Class_file is

  procedure initialize( the:in out File ) is
  begin
    the.state       := not_ACTIVE;    -- Not active
    the.file_length := 0;             -- No file registered
    the.file        := (others=>' '); -- Blank file name
  end initialize;

  procedure finalize( the:in out File ) is
  begin
    if the.state = ACTIVE then
      null;                      -- Dilemma should do something
    end if;
  end finalize;

  procedure register( the:in out File; str:in String ) is
  begin
    the.file( 1 .. str'Length ) := str;
    the.file_length             := str'Length;
    the.state                   := ACTIVE;
  end register;

  function  is_active( the:in File ) return Boolean is
  begin
    return the.state = ACTIVE;
  end is_active;

  procedure set_active( the:in out File ) is
  begin
    the.state := ACTIVE;
  end set_active;

  procedure set_not_active( the:in out File ) is
  begin
    the.state := not_ACTIVE;
  end set_not_active;

  procedure read( the:in out File; s:in out Store ) is
    file_in : Text_io.FILE_type;   -- File handle
    row     : Natural;             -- Processing row
    max_size: Natural;             -- Max lines in store
  begin
    open( File => file_in, Mode => IN_FILE,
          Name => the.file(1 .. Integer(the.file_length)));
    row := 1; max_size := deliver_store_size(s);
    while not End_of_file( file_in ) and row <= max_size loop
      set_position( s, row, 1 );
      declare
        ch       : Character;    -- Character read from file
        col      : Natural;      -- Current col position in line
        line_size: Natural;      -- Maximum line size
      begin
        col := 1; line_size := deliver_max_line_size( s, row );
        while not End_of_line( file_in ) loop
          get( file_in, ch );
          if col <= line_size then
            add( s, ch ); col := col + 1;
          end if;
        end loop;
      end;
      skip_line( file_in ); row := row + 1;
    end loop;
    close( file_in );
    set_position( s, 1, 1 ); set_last_line( s, row-1 );
    the.lines_in_file := row-1;
    the.state := not_ACTIVE;              -- Not changed
  end read;

  procedure write( the:in out File; s:in out Store;
                   u:in User ) is
    file_out : Text_io.FILE_type;   -- File handle
    row      : Natural := 1;
  begin
    if the.state = ACTIVE then
      if the.file(1) = ' ' then
        register( the, dialog( u, "File name: ") );
      end if;
      create( File=> file_out, Mode => OUT_FILE,
              Name => the.file(1..Integer(the.file_length)));
      row := 1;
      while row <= no_lines(s) loop
        declare
          l  : Class_Line.Line;             -- Line to output
          ch : Character;                  -- current character
        begin
          l := deliver_line( s, row ); start(l);
          while not end_of_line(l) loop
            ch := get_char( l ); put( file_out, ch );
            next_ch( l );
          end loop;
        end;
        new_line( file_out ); row := row + 1;
      end loop;
      close( file_out );
      the.state := not_ACTIVE;
 -- else
 --   raise Name_error;
    end if;
  end write;

end Class_file;
