-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Simple_io;
Use  Simple_io;
package body Class_line is

  procedure debug( the:in Line ) is
  begin
    put("Line    >");
    put("last ch      :"); put(Integer(the.cur_len),WIDTH=>2);
    put(" Iter pos    :"); put(Integer(the.iter_pos),WIDTH=>2);
    put(" Active col  :"); put(Integer(the.col), WIDTH=>2);
    new_line;
  end debug;

  procedure initialize( the:in out Line ) is
  begin
    --the := new O_Line;               -- Dynamic Store
    clear(the);
  end initialize;

  procedure finalize( the:in out Line ) is
  begin
    null;
  end finalize;

  procedure adjust( the:in out Line ) is
  begin
    null;
  end adjust;

  procedure clear( the:in out Line ) is
  begin
    the.iter_pos := 0;           -- Iterator
    the.cur_len := 0;            -- Empty Line
    the.col := 1;                -- Current position
  end clear;

  procedure start( the:in out Line ) is
  begin
    the.iter_pos := 1;
  end start;

  function  end_of_line( the:in Line) return Boolean is
  begin
    return the.iter_pos > the.cur_len;
  end end_of_line;

  function  get_char( the:in Line ) return Character is
  begin
    return the.chs( the.iter_pos );
  end get_char;

  procedure next_ch( the:in out Line ) is
  begin
    if the.iter_pos <= the.cur_len then
      the.iter_pos := the.iter_pos + 1;
    end if;
  end next_ch;

  procedure add(the:in out Line; where:in Natural;
                ch:in Character) is
  add_at : Line_index;
  begin
    add_at := Line_index( where );
    if the.cur_len < the.chs'Length and then
       add_at <= the.cur_len+1
    then
      for i in reverse add_at .. the.cur_len loop
        the.chs(i+1) := the.chs(i);       -- Make room
      end loop;
      the.cur_len := the.cur_len + 1;     -- Increase length
      the.chs( add_at ) := ch;            -- Insert character
      if add_at < the.chs'Length then     -- New column
        the.col := add_at + 1;
      end if;
    end if;
  end add;

  procedure del( the:in out Line; where:in Natural ) is
    del_at : Line_index;
  begin
    del_at := Line_index( where );
    if del_at <= the.cur_len then         -- Can delete
      the.cur_len := the.cur_len-1;       -- New length
      the.col     := del_at;              -- New current col
      for i in del_at .. the.cur_len loop
        the.chs(i) := the.chs(i+1);       -- Delete ch
      end loop;
    end if;
    if del_at > the.cur_len then          -- New column
      the.col := Line_index'Max(the.cur_len+1, 1 );
    end if;
  end del;

  function  deliver_current_col( the:in Line ) return Natural is
  begin
    return Natural( the.col );            -- Current position
  end deliver_current_col;

  function  deliver_cur_len( the:in Line ) return Natural is
  begin
    return Natural( the.cur_len );        -- Chars in line
  end deliver_cur_len;

  function  deliver_max_line_size( the:in Line ) return Natural is
  begin
    return MAX_CHS;                         -- Max size of line
  end deliver_max_line_size;

end Class_line;
