-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Pack_constants, Simple_io;
use  Pack_constants, Simple_io;
package body Class_store is

  procedure debug( the:in Store ) is
  begin
    put("Store   >");
    put("no_lines     :"); put(Integer(the.no_lines), WIDTH=>2);
    put(" active row  :"); put(Integer(the.row), WIDTH=>2);
    put(" active col  :"); put(Integer(the.col), WIDTH=>2);
    new_line;
    debug( the.lines( Store_index( the.row ) ) );
  end debug;

  procedure initialize( the:in out Store ) is
  begin
--O    for i in Store_range loop
--O      initialize( the.lines(i) );           -- Individual lines
--O    end loop;
    clear(the);                             -- clear Store
    set_last_line( the, 1 );                -- Set the last line
  end initialize;

  procedure finalize( the:in out Store ) is
  begin
    null;
  end finalize;

  procedure clear( the:in out Store ) is
  begin
    for i in Store_range loop
      clear( the.lines(i) );      -- Individual lines
    end loop;
    the.no_lines := 0;            -- Lines stored
    the.row      := 1;            -- Current Line
    the.col      := 1;            -- Current char in line
  end clear;

  procedure add( the:in out Store; row:in Natural;
                 column:in Natural; ch:in Character ) is
    row_is : Store_index := Store_index( row );
  begin
    if row_is > the.no_lines and then row <= MAX_LINES then
      the.no_lines := the.no_lines + 1;
    end if;
    add( the.lines( row_is ), column, ch );
  end add;

  procedure del( the:in out Store; row:in Natural;
                 column:in Natural ) is
  begin
    del( the.lines( Store_index(row) ), column );
  end del;

  procedure add( the:in out Store; ch:in Character ) is
  begin
    add( the, the.row, the.col, ch );
    the.col :=
      deliver_current_col( the.lines(Store_index(the.row)));
  end add;

  procedure del( the:in out Store ) is
  begin
    del( the, the.row, Natural'Max(the.col-1, 1) );    -- at position
    the.col :=
      deliver_current_col( the.lines(Store_index(the.row)));
  end del;

  procedure left_right( the:in out Store; dir:in Character ) is
    length  : Natural;
  begin
    if dir = C_LEFT then                -- move ->
      if the.col > 1 then               -- Can go left
        the.col := the.col-1;
      end if;
    else                                -- move ->
      length :=
        deliver_cur_len( the.lines(Store_index(the.row)) );
      if the.col <= length then         -- Can go right
        the.col := the.col+1;
      end if;
    end if;
  end left_right;

  procedure up_down( the:in out Store; dir:in Character;
                  no_lines:in Natural ) is
    length  : Natural;
  begin
    if dir = C_UP or else dir = C_PAGE_UP then
      if Integer(the.row) - no_lines >= 1 then
        the.row := the.row - no_lines;
      else
        the.row := 1;
      end if;
    else
      if Integer(the.row)+no_lines<=Integer(the.no_lines) then
        the.row := the.row + no_lines;
      else
        the.row := Natural(the.no_lines);
        if the.row < MAX_LINES and then dir = C_DOWN then
          the.row := the.row + 1;       -- Expand by 1 line
        end if;
      end if;
    end if;
    length := deliver_cur_len(the.lines(Store_index(the.row)));
    the.col := Natural'Max( Natural'Min( length, the.col ), 1 );
  end up_down;

  function deliver_line(the:in Store; row:in Natural) return Line is
  begin
    return the.lines( Store_index(row) );  -- The whole line
  end  deliver_line;

  function  no_lines( the:in Store ) return Natural is
  begin
    return Natural(the.no_lines);          -- Lines in buffer
  end no_lines;

  procedure set_position( the:in out Store; row:in Natural;
                          column:in Natural ) is
  begin
    the.col  := column;         -- The new col
    the.row  := row;            -- The new row
  end set_position;

  procedure set_last_line( the:in out Store; row:in Natural ) is
  begin
    the.no_lines := Store_index(row);  -- New last line
  end set_last_line;

  procedure deliver_row_column( the:in Store; row:out Natural;
                                column:out Natural ) is
  begin
    row    := the.row;           -- The current row
    column := the.col;           -- The current col
  end deliver_row_column;

  function  deliver_store_size( the:in Store ) return Natural is
  begin
    return MAX_LINES;            -- Max size of buffer
  end deliver_store_size;

  function  deliver_max_line_size
              ( the:in Store; row:in Natural ) return Natural is
  begin
    return deliver_max_line_size(the.lines(Store_index(row)));
  end deliver_max_line_size;

end Class_store;
