-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


----------------------------------------------------------------------

with Class_line; use Class_line;
with Ada.Finalization;
use  Ada.Finalization;
package Class_store is

  type Store is new Limited_Controlled with private;

  procedure debug( the:in Store );
  procedure initialize( the:in out Store );
  procedure finalize( the:in out Store );
  procedure clear( the:in out Store );

  procedure add( the:in out Store; row:in Natural;
                 column:in Natural; ch:in Character );
  procedure del( the:in out Store; row:in Natural;
                 column:in Natural );
  procedure add( the:in out Store; ch:in Character );
  procedure del( the:in out Store );

  procedure left_right( the:in out Store; dir:in Character );
  procedure up_down( the:in out Store; dir:in Character;
                     no_lines:in Natural );

  function  deliver_line(the:in Store; row:in Natural) return Line;

  function  no_lines( the:in Store ) return Natural;
  procedure set_position( the:in out Store; row:in Natural;
                          column:in Natural );
  procedure set_last_line( the:in out Store; row:in Natural );
  procedure deliver_row_column( the:in Store; row:out Natural;
                                column:out Natural );
  function  deliver_store_size( the:in Store ) return Natural;
  function  deliver_max_line_size
              ( the:in Store; row:in Natural ) return Natural;

private
  MAX_LINES : CONSTANT := 120;
  type    Store_index    is range 0 .. MAX_LINES;
  subtype Store_range    is Store_index range 1 .. MAX_LINES;
  type    Store_as_array is array ( Store_index ) of Line;

  type Store is new Limited_Controlled with record
     lines     : Store_as_array;    -- Store as array
     no_lines  : Store_index := 0;  -- Lines stores
     row       : Natural := 0;      -- Current row
     col       : Natural := 0;      -- Current column
  end record;
end Class_store;
