-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Ada.Text_io, Ada.Characters.handling;
use  Ada.Text_io, Ada.Characters.handling;
package body Class_line is
  procedure get_line( the:in out Line; fd:in Ada.Text_io.File_type ) is
    pos : Line_index := 0;
    ch  : Character;
  begin
    while not end_of_line( fd ) loop
      get( fd, ch );
      if pos < MAX_LINE then
        pos := pos + 1;
        the.chs(pos) := ch;
      end if;
    end loop;
    the.len := pos;
    skip_line( fd );
  end get_line;
     
  procedure put_line( the:in Line; fd:in Ada.Text_io.File_type ) is
  begin
    if the.open then
      for i in 2 .. the.len loop
        put( fd, the.chs(i) );
      end loop;
      new_line( fd );
    end if;
  end put_line;

  function status( the:in Line ) return Line_status is
    pos  : Line_index := 0;
  begin
    if the.len >= 1 and then the.chs(1) = '+' then 
      return TEXT_LINE;
    end if;
    if the.len >= 2 and then the.chs(1..2) = "@@" then
      for i in 3 .. the.len-2 loop
        if is_upper( the.chs(i) ) or is_lower( the.chs(i) ) or
           is_digit( the.chs(i) ) or the.chs(i) = '_' or 
           the.chs(i) = '.' then 
          pos := pos + 1;
          name(pos) := the.chs(i);
        end if;
      end loop;
      name_pos := pos;
      put( "Extracting file " & name(1..pos) ); new_line;
      return FILE_NAME;
    end if;
    return UNKNOWN;
  end status;
      
  procedure get_fd( the:in out Line; fd:in out Ada.Text_io.File_type ) is
  begin
    if the.open then                    -- Output file open
      close( fd ); the.open := FALSE;
    end if;
    create( File=>fd, Mode=>OUT_FILE,   -- Create file
            Name=>name(1..name_pos) );
    the.open := TRUE;
  exception
    when Name_error =>
      put("Exp: Can not create file " & name(1..name_pos) ); 
      new_line;
    when Status_error =>
      put("Exp: " & name(1..name_pos) & " all ready open" );
      new_line;
    when others =>
      put("Exp: " & name(1..name_pos) & " unknown error" );
      new_line;
  end;

end Class_line;
