-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


with Simple_io, Pack_screen;
use  Simple_io, Pack_screen;
package body Class_board is

  procedure next( the:in Board; x_co,y_co:in out Board_index;
                  dir:in Natural; res:out Boolean);
  function find_turned( the:in Board; x,y: in Board_index )
                        return Natural;
  procedure turn_counters(the: in out Board; x,y: in Board_index;
                          total: out Natural );
  function  no_turned(the:in Board; o_x,o_y:in Board_index;
                      dir:in Natural;
                      n:in Natural := 0 ) return Natural;
  procedure capture(the:in out Board; x_co, y_co:in Board_index;
                    dir:in Natural );

  procedure set_up( the:in out Board ) is
    black_counter: Counter;               -- A white counter
    white_counter: Counter;               -- A black counter
  begin
    set( black_counter, BLACK );          -- Set black
    set( white_counter, WHITE );          -- Set white
    for x in the.sqrs'Range(1) loop
      for y in the.sqrs'Range(2) loop
        initialize( the.sqrs(x,y) );      -- To empty
      end loop;
    end loop;
    add( the.sqrs( SIZE/2,   SIZE/2 ),   black_counter );
    add( the.sqrs( SIZE/2,   SIZE/2+1 ), white_counter );
    add( the.sqrs( SIZE/2+1, SIZE/2 ),   white_counter );
    add( the.sqrs( SIZE/2+1, SIZE/2+1 ), black_counter );
    the.score( BLACK ) := 2; the.score( WHITE ) := 2;
  end set_up;

  procedure add( the:in out Board; x,y:in Integer;
                 move_is:in Move_status ) is
    plays_with: Counter;          -- Current player's counter
    turned    : Natural;          -- Number counters turned
  begin
    set( plays_with, the.player );          -- Set current colour
    the.last_move( the.player ) := move_is; -- Last move is
    if move_is = OK then                    -- Not Pass
      turn_counters(the, x,y, turned);      -- and flip
      add( the.sqrs( x, y ), plays_with );  -- to board
      the.score( the.player ) :=
                 the.score( the.player ) + turned + 1;
      the.score( the.opponent ):=
                 the.score( the.opponent ) - turned;
    end if;
  end add;

  procedure now_playing(the:in out Board; c:in Counter_colour) is
  begin
    the.player   := c;                       -- Player
    case c is                                -- Opponent
       when WHITE => the.opponent := BLACK;
       when BLACK => the.opponent := WHITE;
    end case;
  end now_playing;

  procedure display( the:in Board ) is
    dashes: String( 1 .. the.sqrs'Length*4+1 ) := (others=>'-');
  begin
    screen_clear;                            -- Clear screen
    put( dashes ); new_line;                 -- Top
    for x in the.sqrs'Range(1) loop
      put("|");                              -- Cells on line
      for y in the.sqrs'Range(2) loop
        put(" "); display( the.sqrs(x,y) ); put(" |");
      end loop;
      new_line; put( dashes ); new_line;     -- Bottom lines
    end loop;
    new_line;
    put( "Player X has " );
    put( Integer(the.score(BLACK)), width=>2 );
    put( " counters" ); new_line;
    put( "Player O has " );
    put( Integer(the.score(WHITE)), width=>2 );
    put( " counters" ); new_line;
  end display;

  function check_move( the:in Board; x,y:in Integer )
                       return Move_status is
  begin
    if x = 0 and then y = 0 then
      return PASS;
    elsif x in Board_index and then y in Board_index then
      if holds( the.sqrs( x, y ) ) = EMPTY then
        if find_turned(the, x, y) > 0 then
          return OK; 
        end if;
      end if;
    end if;
    return INVALID;
  end check_move;

  function find_turned( the:in Board; x,y: in Board_index )
                        return Natural is
    sum     : Natural := 0;       -- Total stones turned
  begin
    if holds( the.sqrs( x, y ) ) = EMPTY then
      for dir in 1 .. 8 loop      -- The  8 possible directions
        sum := sum + no_turned( the, x, y, dir );
      end loop;
    end if;
    return sum;                   -- return total
  end find_turned;

  procedure turn_counters(the: in out Board; x,y: in Board_index;
                          total: out Natural ) is
    num_cap  : Natural := 0;
    captured : Natural;
  begin
    if holds( the.sqrs( x, y ) ) = EMPTY then
      for dir in 1 .. 8 loop
        captured := no_turned( the, x, y, dir );
        if captured > 0 then
          capture( the, x, y, dir );
          num_cap := num_cap + captured;
        end if;
      end loop;
    end if;
    total := num_cap;
  end turn_counters;

  function  no_turned(the:in Board; o_x,o_y:in Board_index;
                      dir:in Natural;
                      n:in Natural := 0 ) return Natural is
    ok : Boolean;                           -- Result from next
    nxt: Cell_holds;                        -- Next in line is
    col: Counter_colour;                    -- Counter colour
    x  : Board_index := o_x;                -- Local copy
    y  : Board_index := o_y;                -- Local copy
  begin
    next( the, x,y, dir, ok );              -- Next cell
    if ok then                              -- On the board
      nxt := holds( the.sqrs(x,y) );        -- Contents are
      if nxt = EMPTY then                   -- End of line
        return 0;
      else
        col := to_colour( nxt );            -- Colour
        if col = the.opponent then          -- Opponents counter
          return no_turned(the, x,y, dir, n+1); -- Try next cell
        elsif col = the.player then         -- End of counters
          return n;                         -- Counters turned
        end if;
      end if;
    else
      return 0;                             -- No line
    end if;
  end no_turned;

  procedure next( the:in Board; x_co,y_co:in out Board_index;
                  dir:in Natural; res:out Boolean) is
    x, y   : Natural;
  begin
    x := x_co; y := y_co;         -- May go outside Board_range
    case dir is
      when 1 =>         y:=y+1;   --  Direction to move
      when 2 => x:=x+1; y:=y+1;   --      8   1   2
      when 3 => x:=x+1;           --
      when 4 => x:=x+1; y:=y-1;   --      7   *   3
      when 5 =>         y:=y-1;   --
      when 6 => x:=x-1; y:=y-1;   --      6   5   4
      when 7 => x:=x-1;           --
      when 8 => x:=x-1; y:=y+1;   --
      when others => raise Constraint_error;
    end case;
    if x in Board_range and then y in Board_range then
      x_co :=  x; y_co :=  y;     --
      res := TRUE;                -- Found a next cell
    else
      res := FALSE;               -- No next cell
    end if;
  end next;

  procedure capture(the:in out Board; x_co, y_co:in Board_index;
             dir:in Natural ) is
    ok   : Boolean;                   -- There is a next cell
    x, y : Board_index;               -- Coordinates of cell
    nxt  : Cell_holds;                -- Next in line is
  begin
    x := x_co; y := y_co;
    next( the, x, y, dir, ok );       -- Calculate pos next cell
    if ok then                        -- Cell exists (Must)
      nxt := holds( the.sqrs(x,y) );
      if to_colour( nxt ) = the.opponent then
        flip( the.sqrs(x, y) );       -- Capture
        capture(the, x, y, dir );     -- Implement capture
      else
        return;                       -- End of line
      end if;
    else
      raise Constraint_error;         -- Will never occur
    end if;
  end capture;

  function  status ( the:in Board ) return State_of_game is
  begin
    if the.score( the.opponent ) = 0 then
      return WIN;
    end if;
    if (the.sqrs'Length(1) * the.sqrs'Length(2) =
        the.score(the.opponent)+the.score(the.player)) or
       (the.last_move(BLACK)=PASS and the.last_move(WHITE)=PASS)
    then
      if the.score(the.opponent) = the.score(the.player)
         then return DRAW;
      end if;
      if the.score(the.opponent) < the.score(the.player)
         then return WIN;
      else
        return LOSE;
      end if;
    end if;
    return PLAY;
  end;

  function contents( the:in Board; x,y:in Integer )
                     return Cell_holds is
  begin
      return holds( the.sqrs( x, y ) );
  end contents;

end Class_board;
