-------------------------------------------------------------
-- (C) Michael A Smith 1993-1997                           --
-- Taken from the book Object-oriented software in Ada 95  --
--  Published by International Thomson Publishing          --
--     See http://www.brighton.ac.uk/ada95/home.htm       --
-------------------------------------------------------------


----------------------------------------------------------------------

with Class_counter, Class_cell;
use  Class_counter, Class_cell;
package Class_board is

  type Board         is private;
  type State_of_game is ( PLAY, WIN, DRAW, LOSE );
  type Move_status   is ( OK, INVALID, PASS );

  procedure set_up( the:in out Board );
  procedure add( the:in out Board; x,y:in Integer;
                 move_is:in Move_status );
  procedure now_playing( the:in out Board; c:in Counter_colour );
  procedure display( the:in Board );
  function  check_move( the:in Board; x,y:in Integer )
                        return Move_status;
  function  status( the:in Board ) return State_of_game;
  function  contents( the:in Board; x,y:in Integer )
                     return Cell_holds;

private
  SIZE: CONSTANT := 8;                            -- 8 * 8 Board
  subtype Board_index is Integer range 1 .. SIZE; --
  subtype Board_range is Board_index;             --

  type Board_array is array (Board_range, Board_range) of Cell;
  type Score_array is array (Counter_colour) of Natural;
  type Move_array  is array (Counter_colour) of Move_status;

  type Board is record
    sqrs     : Board_array;               -- Game board
    player   : Counter_colour;            -- Current Player
    opponent : Counter_colour;            -- Opponent
    score    : Score_array;               -- Running score
    last_move: Move_array;                -- Last move is
  end record;
end Class_board;
