	/************************************************/
	/*						*/
	/*			RatC			*/
	/*						*/
	/*		Lancaster implementation	*/
	/*			by			*/
	/*	    Bob Berry and Brain Meekings	*/
	/*			May 1983		*/
	/*						*/
	/*	(based on Ron Cain's	Small-C)	*/
	/*						*/
	/************************************************/


	/* implementation dependent definitions */

#include	<stdio.h>		/* UNIX I/O	*/
#define		intwidth	2	/* integer size	*/
#define		charwidth	1	/* char size	*/
#define		clearscreen	{putchar(27);putchar('E');}

	/* Ascii codes */

#define		bspch		8
#define		tabch		9
#define		eolch		10
#define		ffch		12
#define		crch		13
#define		quoch		39
#define		bslch		92


	/* Output definitions */

#define		nl		outbyte(eolch)
#define		tab		outbyte(tabch)
#define		colon		outbyte(':')
#define		comma		outbyte(',')
#define		space		outbyte(' ')
#define		comment		outbyte(';')


	/* Define the symbol table parameters */

#define		symsiz		15
#define		symtbsz		5400
#define		numglbs		300
#define		startglb	symtab
#define		endglb		startglb+numglbs*symsiz
#define		startloc	endglb+symsiz
#define		endloc		symtab+symtbsz-symsiz


	/* Define symbol table entry format */

#define		name		0
#define		ident		10
#define		type		11
#define		storage		12
#define		offset		13

	/* System wide name size (for symbols) */

#define		namesize	10
#define		namemax		9


	/* Define possible entries for "ident" */

#define		variable	1
#define		array		2
#define		pointer		3
#define		function	4


	/* Define possible entries for "type" */

#define		cchar		1
#define		cint		2


	/* Define possible entries for "storage" */

#define		statik		1
#define		stkloc		2


	/* Define the "while" statement queue */

#define		wqtabsz		100
#define		wqsiz		4
#define		wqmax		wq+wqtabsz-wqsiz


	/* Define entry offsets in while queue */

#define		wqsym		0
#define		wqsp		1
#define		wqloop		2
#define		wqlab		3


	/* Define the literal pool */

#define		litabsz		2500
#define		litmax		litabsz-1


	/* Define the input line */

#define		linesize	80
#define		linemax		linesize-1


	/* Define the macro (define) pool */

#define		macbsize	1500
#define		mactsize	75
#define		macssize	750	/*namesize*mactsize*/


	/* Define statement types (tokens) */

#define		stif		1
#define		stwhile		2
#define		streturn	3
#define		stbreak		4
#define		stcont		5
#define		stasm		6
#define		stexp		7


	/* Now reserve some storage words */

char	symtab[symtbsz];	/* symbol table		*/
char	*glbptr,*locptr;	/* ptrs to next entries */
int	wq[wqtabsz];		/* while queue		*/
int	*wqptr;			/* ptr to next entry	*/

char	litq[litabsz];		/* literal pool		*/
int	litptr;			/* ptr to next entry	*/

char	macb[macbsize];		/* macro body buffer	*/
char	mact[macssize];		/* macro name table	*/
char	*macbptr,*mactptr;	/* and their indices	*/
char	*macbmax;		/* end of body buffer	*/
int	macp[mactsize];		/* ptrs into body	*/
int	macpmax,*macpptr;	/* macro count		*/

char	line[linesize];		/* parsing buffer	*/
char	mline[linesize];	/* temp macro buffer	*/
char	*ch,*nexch,*chmax,*mptr; /* ptrs into each	*/
int	mpmax;			/* limit of mline	*/


	/* Miscellaneous storage */

int	nxtlab,		/* next avialable label #	*/
	litlab,		/* label # assigned to lit pool */
	sp,		/* stack pointer		*/
	argstk,		/* function argument sp		*/
	ncmp,		/* # open compound statements	*/
	errcnt,		/* # errors in compilation	*/
	inscnt,		/* # instructions generated	*/
	lncnt,		/* # source lines		*/
	eof,		/* non-zero on final input eof	*/
	input,		/* iob # for input file		*/
	output,		/* iob # for output file	*/
	input2,		/* iob # for "include" file	*/
	glbflag,	/* non-zero if internal globals */
	mulfile,	/* non-zero for many input files*/
	ctext,		/* non-zero to intermix c-source*/
	cmode,		/* non-zero while parsing c-code*/
			/* zero while parsing assembly	*/
	lastst;		/* last executed statement type	*/

char	quote[2];	/* literal string for '"'	*/
char	*cptr;		/* work ptr to any char buffer	*/
int	hasmain;	/* does this file have "main()"	*/



	/************************************************/
	/*						*/
	/*	  Complier begins execution here	*/
	/*						*/
	/************************************************/

main()
{
	glbptr=startglb;	/* clear global symbols	*/
	locptr=startloc;	/* clear local symbols	*/
	chmax =line+linemax;	/* max value of ptr ch	*/
	mpmax =mline+linemax;	/* ditto for mline	*/
	wqptr=wq;		/* clear while queue	*/

	litptr=			/* clear literal pool	*/
	sp=			/* stack ptr (relative)	*/
	eof=			/* not eof yet		*/
	input=			/* no input file	*/
	input2=			/* or include file	*/
	output=			/* no open units	*/
	ncmp=			/* no open compounds	*/
	lastst=			/* no last stmt yet	*/
	0;			/* ... all set to zero	*/

	errcnt=			/* no errors		*/
	inscnt=			/* no instructions yet	*/
	lncnt=			/* no source lines yet	*/
	hasmain=		/* no main segement yet	*/
	0;			/* ... all set to zero	*/

	mactptr=mact;		/* clear the macro table*/
	macbptr=macb;		/* start of body buffer	*/
	macpptr=macp;		/* none to start with	*/
	macpmax=macp+(mactsize-1)*intwidth;
	macbmax=macb+macbsize-1;

	quote[0]='"';
	quote[1]=0;		/* fake a quote literal	*/
	cmode=1;		/* enable preprocessing	*/

	/************************/
	/*    Compiler body	*/
	/************************/

	ask();			/* get user options	*/
	openout();		/* get an output file	*/
	openin();		/* & initial input file	*/
	comment;
	outstr(" Lancaster RatC compiler");
	nl;
	parse();		/* process ALL input	*/
	dumplits();		/* dump literal pool	*/
	dumpglbs();		/* and all static memory*/
	trailer();		/* follow up code	*/
	closeout();		/* close the output	*/
	errorsummary();		/* summarize errors	*/
	return;			/* then exit to system	*/
}

	/********************************/
	/*	Process all input text	*/
	/*  At this level, only static	*/
	/*	declarations, defines	*/
	/*	includes and function	*/
	/*	definitions are legal	*/
	/********************************/

parse()
{
	while (eof==0)			/* do until no more input */
	{
		if (amatch("char",4))
			{declglb(cchar);needsemi();}
		else if (amatch("int",3))
			{declglb(cint);needsemi();}
		else if (match("#asm"))
			doasm();
		else if (match("#include"))
			openinclude();
		else if (match("#define"))
			addmac();
		else newfunc();
		skipblanks();		/* force eof if pending */
	}
}

	/****************************************/
	/*	New function definition		*/
	/****************************************/

newfunc()
{
	char n[namesize],*ptr;
	int argtop;

	if (symname(n)==0)
	{
		error("illegal function or declaration");
		resetptr();
		return;
	}
	if (ptr=findglb(n))
	{
		if (ptr[ident]!=function) errmulti(n);
		else if (ptr[offset]==function) errmulti(n);
		else ptr[offset]=function;
	}
	else addglb(n,function,cint,function);

	if (match("(")==0)
		error("missing opening parenthesis");
	if (astreq(n+1,"main",4))
		{ hasmain=1;header();}
	outstr("_");outstr(n);colon;nl;

	locptr=startloc;
	argstk=0;
	while (match(")")==0)
	{
		if (symname(n))
		{
			if (findloc(n)) errmulti(n);
			else
			{
				addloc(n,0,0,argstk);
				argstk=argstk+intwidth;
			}
		}
		else { error("illegal argument name");skipchars();}
		skipblanks();
		if (streq(ch,")")==0)
		{
			if (match(",")==0) error("expected comma");
		}
		if (needstend()) break;
	}
	sp=0;argtop=argstk;
	while (argstk)
	{
		if (amatch("char",4))
			{ getarg(cchar,argtop); needsemi();}
		else if (amatch("int",3))
			{ getarg(cint,argtop); needsemi();}
		else { error("wrong number of args"); break;}
	}
	if (statement()!=streturn)
	{
		modstk(0);
		ret();
	}
	sp=0;
	locptr=startloc;
}

	/********************************/
	/*	Get function arguments	*/
	/********************************/

getarg(t,argtop)
int t,argtop;
{
	int j,legalname,address;
	char n[namesize],c,*argptr;

	while(1)
	{
		if (argstk==0) return;
		if (match("*")) j=pointer; else j=variable;
		if ((legalname=symname(n))==0) errname();
		if (match("["))
		{
			while (inbyte()!=']')
				if (needstend()) break;
			j=pointer;
		}
		if (legalname)
		{
			if (argptr=findloc(n))
			{
				argptr[ident]=j;
				argptr[type]=t;
				address=argtop-((argptr[offset]&255)+
					((argptr[offset+1]&255)<<8));
				argptr[offset]=address;
				argptr[offset+1]=address>>8;
			}
			else error("expecting argument name");
		}
		argstk=argstk-intwidth;
		if (needstend()) return;
		if (match(",")==0) error("expected comma");
	}
}

	/********************************/
	/*     Process a statement	*/
	/********************************/

statement()
{
	if ((*ch==0) & (eof)) return;
	else if (amatch("char",4))
		{ declloc(cchar); needsemi(); }
	else if (amatch("int",3))
		{ declloc(cint); needsemi(); }
	else if (match("{"))
		compound();
	else if (amatch("if",2))
		{ doif(); lastst=stif; }
	else if (amatch("while",5))
		{ dowhile(); lastst=stwhile; }
	else if (amatch("return",6))
		{ doreturn(); needsemi(); lastst=streturn; }
	else if (amatch("break",5))
		{ dobreak(); needsemi(); lastst=stbreak; }
	else if (amatch("continue",8))
		{ docont(); needsemi(); lastst=stcont; }
	else if (match(";"));
	else if (match("#asm"))
		{ doasm(); lastst=stasm; }
	else	{expression(); needsemi(); lastst=stexp; }
	return lastst;
}

	/********************************/
	/*  Process compound statement	*/
	/********************************/

compound()
{
	++ncmp;
	while (match("}")==0)
		if (eof) return; else statement();
	--ncmp;
}

	/********************************/
	/*	Process IF statement	*/
	/********************************/

doif()
{
	int flev,fsp,flab1,flab2;

	flev=locptr;
	fsp=sp;
	flab1=getlabel();
	test(flab1);
	statement();
	sp=modstk(fsp);
	locptr=flev;
	if (amatch("else",4)==0)
	{
		outlabel(flab1); colon; nl;
		return;
	}
	jump(flab2=getlabel());
	outlabel(flab1); colon; nl;
	statement();
	sp=modstk(fsp);
	locptr=flev;
	outlabel(flab2); colon; nl;
}

	/********************************/
	/*   Process WHILE statement	*/
	/********************************/

dowhile()
{
	int wq[4];

	wq[wqsym]=locptr;
	wq[wqsp]=sp;
	wq[wqloop]=getlabel();
	wq[wqlab]=getlabel();
	addwhile(wq);
	outlabel(wq[wqloop]); colon; nl;
	test(wq[wqlab]);
	statement();
	jump(wq[wqloop]);
	outlabel(wq[wqlab]); colon; nl;
	locptr=wq[wqsym];
	sp=modstk(wq[wqsp]);
	delwhile();
}

	/********************************/
	/*   Process RETURN statement	*/
	/********************************/

doreturn()
{
	if (needstend()==0) expression();
	modstk(0);
	ret();
}

	/********************************/
	/*    Process BREAK statement	*/
	/********************************/

dobreak()
{
	int *ptr;

	if ((ptr=readwhile())==0) return;
	modstk((ptr[wqsp]));
	jump(ptr[wqlab]);
}

	/********************************/
	/*  Process CONTINUE statement	*/
	/********************************/

docont()
{
	int *ptr;

	if ((ptr=readwhile())==0) return;
	modstk((ptr[wqsp]));
	jump(ptr[wqloop]);
}

	/********************************/
	/*   Process #asm directive	*/
	/********************************/

doasm()
{
	cmode=0;
	while(1)
	{
		inline();
		if (match("#endasm")) break;
		if (eof) break;
		outstr(line);
		nl;
	}
	resetptr();
	cmode=1;
}

	/********************************/
	/*    Expression evaluation by	*/
	/*	recursive descent	*/
	/********************************/

expression()
{
	int lval[2];

	if (hier1(lval)) rvalue(lval);
}

hier1(lval)
int lval[];
{
	int k,lval2[2];

	k=hier2(lval);
	if (match("="))
	{
		if (k==0) {errlval();return 0;}
		if (lval[1]) push();
		if (hier1(lval2)) rvalue(lval2);
		store(lval);
		return 0;
	}
	else return k;
}

hier2(lval)
int lval[];
{
	int k,lval2[2];

	k=hier3(lval);
	skipblanks();
	if (*ch!='|') return k;
	if (k) rvalue(lval);
	while (1)
	{
		if (match("|"))
		{
			push();
			if (hier3(lval2)) rvalue(lval2);
			pop();
			or();
		}
		else return 0;
	}
}

hier3(lval)
int lval[];
{
	int k,lval2[2];

	k=hier4(lval);
	skipblanks();
	if (*ch!='^') return k;
	if (k) rvalue(lval);
	while (1)
	{
		if (match("^"))
		{
			push();
			if (hier4(lval2)) rvalue(lval2);
			pop();
			xor();
		}
		else return 0;
	}
}

hier4(lval)
int lval[];
{
	int k,lval2[2];

	k=hier5(lval);
	skipblanks();
	if (*ch!='&') return k;
	if (k) rvalue(lval);
	while (1)
	{
		if (match("&"))
		{
			push();
			if (hier5(lval2)) rvalue(lval2);
			pop();
			and();
		}
		else return 0;
	}
}

hier5(lval)
int lval[];
{
	int k,lval2[2];

	k=hier6(lval);
	skipblanks();
	/* check for == and != */
	if (*nexch !='=') return k;
	if ((*ch!='=')&(*ch!='!')) return k;
	if (k) rvalue(lval);
	while (1)
	{
		if (match("=="))
		{
			push();
			if (hier6(lval2)) rvalue(lval2);
			pop();
			eq();
		}
		else if (match("!="))
		{
			push();
			if (hier6(lval2)) rvalue(lval2);
			pop();
			ne();
		}
		else return 0;
	}
}

hier6(lval)
int lval[];
{
	int k,lval2[2];

	k=hier7(lval);
	skipblanks();
	/* wish to identify >,<,>=,<= but reject >>,<< */
	if ((*ch!='<')&(*ch!='>')) return k;
	if ((*nexch=='<')|(*nexch=='>')) return k;
	if (k) rvalue(lval);
	while (1)
	{
		if (match("<="))
		{
			push();
			if (hier7(lval2)) rvalue(lval2);
			pop();
			if (cptr=lval[0])
				if (cptr[ident]==pointer)
				{
					ule();
					continue;
				}
			if (cptr=lval2[0])
				if (cptr[ident]==pointer)
				{
					ule();
					continue;
				}
			le();
		}
		else if (match(">="))
		{
			push();
			if (hier7(lval2)) rvalue(lval2);
			pop();
			if (cptr=lval[0])
				if (cptr[ident]==pointer)
				{
					uge();
					continue;
				}
			if (cptr=lval2[0])
				if (cptr[ident]==pointer)
				{
					uge();
					continue;
				}
			ge();
		}
		else if ((streq(ch,"<"))&(streq(ch,"<<")==0))
		{
			inbyte();
			push();
			if (hier7(lval2)) rvalue(lval2);
			pop();
			if (cptr=lval[0])
				if (cptr[ident]==pointer)
				{
					ult();
					continue;
				}
			if (cptr=lval2[0])
				if (cptr[ident]==pointer)
				{
					ult();
					continue;
				}
			lt();
		}
		else if ((streq(ch,">"))&(streq(ch,">>")==0))
		{
			inbyte();
			push();
			if (hier7(lval2)) rvalue(lval2);
			pop();
			if (cptr=lval[0])
				if (cptr[ident]==pointer)
				{
					ugt();
					continue;
				}
			if (cptr=lval2[0])
				if (cptr[ident]==pointer)
				{
					ugt();
					continue;
				}
			gt();
		}
		else return 0;
	}
}

hier7(lval)
int lval[];
{
	int k,lval2[2];

	k=hier8(lval);
	skipblanks();
	if ((streq(ch,">>")==0)&(streq(ch,"<<")==0)) return k;
	if (k) rvalue(lval);
	while (1)
	{
		if (match(">>"))
		{
			push();
			if (hier8(lval2)) rvalue(lval2);
			pop();
			asr();
		}
		else if (match("<<"))
		{
			push();
			if (hier8(lval2)) rvalue(lval2);
			pop();
			asl();
		}
		else return 0;
	}
}

hier8(lval)
int lval[];
{
	int k,lval2[2];

	k=hier9(lval);
	skipblanks();
	if ((*ch!='+')&(*ch!='-')) return k;
	if (k) rvalue(lval);
	while (1)
	{
		if (match("+"))
		{
			push();
			if (hier9(lval2)) rvalue(lval2);
			if (cptr=lval[0])
				if ((cptr[ident]==pointer)&
					(cptr[type]==cint)) scale(intwidth);
			pop();
			add();
		}
		else if (match("-"))
		{
			push();
			if (hier6(lval2)) rvalue(lval2);
			if (cptr=lval[0])
				if ((cptr[ident]==pointer)&
					(cptr[type]==cint)) scale(intwidth);
			pop();
			sub();
		}
		else return 0;
	}
}

hier9(lval)
int lval[];
{
	int k,lval2[2];

	k=hier10(lval);
	skipblanks();
	if ((*ch!='*')&(*ch!='/')&(*ch!='%')) return k;
	if (k) rvalue(lval);
	while (1)
	{
		if (match("*"))
		{
			push();
			if (hier10(lval2)) rvalue(lval2);
			pop();
			mult();
		}
		else if (match("/"))
		{
			push();
			if (hier10(lval2)) rvalue(lval2);
			pop();
			div();
		}
		else if (match("%"))
		{
			push();
			if (hier10(lval2)) rvalue(lval2);
			pop();
			mod();
		}
		else return 0;
	}
}

hier10(lval)
int lval[];
{
	int k;
	char *ptr;

	if (match("++"))
	{
		if ((k=hier10(lval))==0) {errlval(); return 0;}
		if (lval[1]) push();
		rvalue(lval);
		ptr=lval[0];
		if ((ptr[ident]==pointer)&(ptr[type]==cint))
			inc(intwidth);
		else inc(charwidth);
		store(lval);
		return 0;
	}
	else if (match("--"))
	{
		if ((k=hier10(lval))==0) {errlval(); return 0;}
		if (lval[1]) push();
		rvalue(lval);
		ptr=lval[0];
		if ((ptr[ident]==pointer)&(ptr[type]==cint))
			dec(intwidth);
		else dec(charwidth);
		store(lval);
		return 0;
	}
	else if (match("-"))
	{
		k=hier10(lval);
		if (k) rvalue(lval);
		neg();
		return 0;
	}
	else if (match("*"))
	{
		k=hier10(lval);
		if (k) rvalue(lval);
		lval[1]=cint;
		if (ptr=lval[0]) lval[1]=ptr[type];
		lval[0]=0;
		return 1;
	}
	else if (match("&"))
	{
		k=hier10(lval);
		if (k==0) { error("illegal address"); return 0;}
		else if (lval[1]) return 0;
		else
		{
			immed();
			outstr(ptr=lval[0]);
			nl;
			lval[1]=ptr[type];
			return 0;
		}
	}
	else
	{
		k=hier11(lval);
		if (match("++"))
		{
			if (k==0) {errlval();return 0;}
			if (lval[1]) push();
			rvalue(lval);
			ptr=lval[0];
			if ((ptr[ident]==pointer)&(ptr[type]==cint))
				inc(intwidth);
			else inc(charwidth);
			store(lval);
			if ((ptr[ident]==pointer)&(ptr[type]==cint))
				dec(intwidth);
			else dec(charwidth);
			return 0;
		}
		else if (match("--"))
		{
			if (k==0) {errlval();return 0;}
			if (lval[1]) push();
			rvalue(lval);
			ptr=lval[0];
			if ((ptr[ident]==pointer)&(ptr[type]==cint))
				dec(intwidth);
			else dec(charwidth);
			store(lval);
			if ((ptr[ident]==pointer)&(ptr[type]==cint))
				inc(intwidth);
			else inc(charwidth);
			return 0;
		}
		else return k;
	}
}

hier11(lval)
int *lval;
{
	int k;
	char *ptr;

	k=primary(lval);
	ptr=lval[0];
	skipblanks();
	if ((*ch=='[')|(*ch=='('))
	while(1)
	{
		if (match("["))
		{
			if (ptr==0)
			{
				error("can't subscript");
				skipchars();
				needbrack("]");
				return 0;
			}
			else if (ptr[ident]==pointer)
				rvalue(lval);
			else if (ptr[ident]!=array)
			{
				error("can't subscript");
				k=0;
			}
			push();
			expression();
			needbrack("]");
			if (ptr[type]==cint) scale(intwidth);
			pop();
			add();
			lval[0]=0;
			lval[1]=ptr[type];
			k=1;
		}
		if (match("("))
		{
			if (ptr==0) callfunction(0);
			else if (ptr[ident]!=function)
			{
				rvalue(lval);
				callfunction(0);
			}
			else callfunction(ptr);
			k=lval[0]=0;
		}
		else return k;
	}
	if (ptr==0) return k;
	if (ptr[ident]==function)
	{
		immed();
		outstr(ptr);
		nl;
		return 0;
	}
	return k;
}

primary(lval)
int *lval;
{
	char *ptr,sname[namesize];
	int num[1];
	int k;

	if (match("("))
	{
		k=hier1(lval);
		needbrack(")");
		return k;
	}
	if (symname(sname))
	{
		if (ptr=findloc(sname))
		{
			getloc(ptr);
			lval[0]=ptr;
			lval[1]=ptr[type];
			if (ptr[ident]==pointer) lval[1]=cint;
			if (ptr[ident]==array) return 0; else return 1;
		}
		if (ptr=findglb(sname)) if (ptr[ident]!=function)
			{
				lval[0]=ptr;
				lval[1]=0;
				if (ptr[ident]!=array) return 1;
				immed();
				outstr(ptr);
				nl;
				lval[1]=ptr[type];
				return 0;
			}
		ptr=addglb(sname,function,cint,0);
		lval[0]=ptr;
		lval[1]=0;
		return 0;
	}
	if (constant(num)) return(lval[0]=lval[1]=0);
	else
	{
		error("invalid expression");
		immed();
		outdec(0);
		nl;
		skipchars();
		return 0;
	}
}

	/********************************/
	/*   Process function call	*/
	/********************************/

callfunction(ptr)
char *ptr;
{
	int nargs;

	nargs=0;
	skipblanks();
	if (ptr==0) push();
	while (*ch!=')')
	{
		if (needstend()) break;
		expression();
		if (ptr==0) swapstk();
		push();
		nargs=nargs+intwidth;
		if (match(",")==0) break;
	}
	needbrack(")");
	if (ptr) call(ptr+1);
	else callstk();
	sp=modstk(sp+nargs);
}

	/********************************/
	/* Declare a static variable -	*/
	/* makes an entry in the symbol */
	/* table so that subsequent	*/
	/* references can call symbol	*/
	/* by name			*/
	/********************************/

declglb(typ)
int typ;
{
	int k,j; char sname[namesize];

	while (1)
	{
		while (1)
		{
			if (needstend()) return;
			k=1;
			if (match("*")) j=pointer;
			else j=variable;
			if (symname(sname)==0) errname();
			if (findglb(sname)) errmulti(sname);
			if (match("["))
			{
				k=needsub();
				if (k) j=array;
				else j=pointer;
			}
			addglb(sname,j,typ,k);
			break;
		}
		if (match(",")==0) return;
	}
}

	/********************************/
	/*    Declare local variables	*/
	/********************************/

declloc(typ)
int typ;
{
	int k,j; char sname[namesize];

	while (1)
	{
		while (1)
		{
			if (needstend()) return;
			if (match("*")) j=pointer;
			else j=variable;
			if (symname(sname)==0) errname();
			if (findloc(sname)) errmulti(sname);
			if (match("["))
			{
				k=needsub();
				if (k)
				{
					j=array;
					if (typ==cint) k=k*intwidth;
				}
				else 
				{
					j=pointer;
					k=intwidth;
				}
			}
			else
				if ((typ==cchar)&(j!=pointer)) k=charwidth;
				else k=intwidth;
			sp=modstk(sp-k);
			addloc(sname,j,typ,sp);
			break;
		}
		if (match(",")==0) return;
	}
}

	/********************************/
	/*  Insert new global symbol	*/
	/********************************/

addglb(sname,id,typ,value)
char *sname,id,typ;
int value;
{
	char *ptr;

	if (cptr=findglb(sname)) return cptr;
	if (glbptr>=endglb)
	{
		error("global symbol table overflow");
		return 0;
	}
	cptr=ptr=glbptr;
	while (*ptr++=*sname++);
	cptr[ident]=id;
	cptr[type]=typ;
	cptr[storage]=statik;
	cptr[offset]=value;
	cptr[offset+1]=value>>8;
	glbptr=glbptr+symsiz;
	return cptr;
}

	/********************************/
	/*   Find a global symbol name	*/
	/********************************/

findglb(sname)
char *sname;
{
	char *ptr;

	ptr=startglb;
	while (ptr!=glbptr)
	{
		if (*sname==*ptr)	/* check lengths */
			if (streq(sname,ptr)) return ptr;
		ptr=ptr+symsiz;
	}
	return 0;
}

	/********************************/
	/*   Insert new local symbol	*/
	/********************************/

addloc(sname,id,typ,value)
char *sname,id,typ;
int value;
{
	char *ptr;

	if (cptr=findloc(sname)) return cptr;
	if (locptr>=endloc)
	{
		error("local symbol table overflow");
		return 0;
	}
	cptr=ptr=locptr;
	while (*ptr++=*sname++);
	cptr[ident]=id;
	cptr[type]=typ;
	cptr[storage]=stkloc;
	cptr[offset]=value;
	cptr[offset+1]=value>>8;
	locptr=locptr+symsiz;
	return cptr;
}

	/********************************/
	/*   Find a local symbol name	*/
	/********************************/

findloc(sname)
char *sname;
{
	char *ptr;

	ptr=startloc;
	while (ptr!=locptr)
	{
		if (*sname==*ptr)	/* check lengths */
			if (streq(sname,ptr)) return ptr;
		ptr=ptr+symsiz;
	}
	return 0;
}

	/********************************/
	/*  Put a new macro definition	*/
	/*	   in the table		*/
	/********************************/

addmac()
{
	char sname[namesize],*sn,*mn;

	if (symname(sname)==0)
	{
		errname();
		resetptr();
		return;
	}
	sn=sname; mn=mactptr;
	if (macpmax>=macpptr)
	{		/* add macro to table */
		while (*mn++ = *sn++);
		*macpptr++=macbptr; mactptr=mactptr+namesize;
	}
	else error("macro count exceeded");
	while (*ch==' '|*ch==tabch) gch();
	while (*macbptr++=gch())	/* add macro body to buffer */
	{
		if (macbptr>=macbmax)
			{error("macro table full");break;}
	}
}

	/********************************/
	/*  Look up posible macro name	*/
	/********************************/

findmac(sname)
char *sname;
{
	int *mqp;
	char *mqt;

	mqp=macp; mqt=mact;
	while (mqp<macpptr)
	{
		if (*sname==*mqt)	/* check lengths */
			if (streq(sname,mqt)) return (*mqp);
		mqt=mqt+namesize; mqp++;
	}
	return 0;
}

	/********************************/
	/*  WHILE table manipulations	*/
	/********************************/

addwhile(ptr)
int ptr[];
{
	int k;

	if (wqptr==wqmax)
		{ error("too many active whiles"); return; }
	k=0;
	while (k<wqsiz)
		{ *wqptr++=ptr[k++]; }
}

delwhile()
{
	if (readwhile()) wqptr=wqptr-wqsiz;
}

readwhile()
{
	if (wqptr==wq)
		{ error("no active whiles"); return 0; }
	else return (wqptr-wqsiz);
}

	/********************************/
	/*	Generate next label	*/
	/********************************/

getlabel()
{
	return(++nxtlab);
}

	/********************************/
	/*	Read symbol name	*/
	/********************************/

symname(sname)
char *sname;
{
	int k;

	skipblanks();
	if (alpha(*ch)==0) return 0;
	k=1;
	while (an(*ch)) sname[k++]=gch();
	sname[k]=0;
	sname[0]=k-1;	/* first char is length of symbol */
	return 1;
}

	/********************************/
	/*  Check for number in input	*/
	/********************************/

number(val)
int val[];
{
	int k,minus;
	char c;

	k=minus=1;
	while (k)
	{
		k=0;
		if (match("+")) k=1;
		if (match("-")) { minus=(-minus); k=1; }
	}
	if (numeric(*ch)==0) return 0;
	while (numeric(*ch))
	{
		c=inbyte();
		k=k*10+(c-'0');
	}
	if (minus<0) k=(-k);
	val[0]=k;
	return 1;
}

	/********************************/
	/*	Load a constant		*/
	/********************************/

constant(val)
int val[];
{
	if (number(val)) immed();
	else if (getqchar(val)) immed();
		else if (getqstring(val))
		{
			immed();
			outlabel(litlab);
			outbyte('+');
		}
		else return 0;
	outdec(val[0]);
	nl;
	return 1;
}

	/********************************/
	/*  Get one or two characters	*/
	/*	from input stream	*/
	/********************************/

getqchar(val)
int val[];
{
	int k;
	char c;

	k=0;
	if (match("'")==0) return 0;
	if ((c=gch())==bslch)	/* escape sequence ? */
	{
		if ((c=gch())=='n') k=eolch;	/* newline	*/
		else if (c=='t') k=tabch;	/* tab		*/
		else if (c=='b') k=bspch;	/* backspace	*/
		else if (c=='r') k=crch;	/* return	*/
		else if (c=='f') k=ffch;	/* form feed	*/
		else if (c=='\\') k=bslch;	/* backslash	*/
		else if (c=='0') k=0;		/* null		*/
		else k=c;
	}
	else k=c;
	if (match("'")==0) return 0;
	val[0]=k;
	return 1;
}

	/********************************/
	/* Get string from input stream	*/
	/********************************/

getqstring(val)
int val[];
{
	char c;

	if (match(quote)==0) return 0;
	val[0]=litptr;
	while (*ch!='"')
	{
		if (*ch==0) break;
		if (litptr>=litmax)
		{
			error("string space exhausted");
			while (match(quote)==0) if (gch()==0) break;
			return 1;
		}
		litq[litptr++]=gch();
	}
	gch();
	litq[litptr++]=0;
	return 1;
}

	/********************************/
	/*	   String compare	*/
	/********************************/

streq(str1,str2)
char *str1,*str2;
{
	int k;

	k=0;
	while (*str2)
	{
		if ((*str1)!=(*str2)) return 0;
		k++; str1++; str2++;
	}
	return k;
}

	/********************************/
	/*  String compare over 'len'	*/
	/*	    charaters		*/
	/********************************/

astreq(str1,str2,len)
char *str1,*str2;
int len;
{
	int k;

	k=0;
	while (k<len)
	{
		if ((*str1)!=(*str2)) break;
		if (*str1==0) break;
		if (*str2==0) break;
		k++; str1++; str2++;
	}
	if (an(*str1)) return 0;
	if (an(*str2)) return 0;
	return k;
}

	/********************************/
	/*  Compare literal with line	*/
	/*  buffer contents, advancing	*/
	/*   buffer pointer if found	*/
	/********************************/

match(lit)
char *lit;
{
	int k;

	skipblanks();
	if (k=streq(ch,lit))
		{ ch=ch+k; nexch=ch+1; return 1; }
	return 0;
}

	/********************************/
	/*  As 'match', but over 'len'	*/
	/*	     characters		*/
	/********************************/

amatch(lit,len)
char *lit;
int len;
{
	int k;

	skipblanks();
	if (k=astreq(ch,lit,len))
	{
		ch=ch+k; nexch=ch+1;
		while (an(*ch)) inbyte();
		return 1;
	}
	return 0;
}

	/********************************/
	/*	Get array bounds	*/
	/********************************/

needsub()
{
	int num[1];

	if (match("]")) return 0;
	if (number(num)==0)
	{
		error("must be constant");
		num[0]=1;
	}
	if (num[0]<0)
	{
		error("negative size illegal");
		num[0]=(-num[0]);
	}
	needbrack("]");
	return num[0];
}

	/********************************/
	/*	Check for semicolon	*/
	/********************************/

needsemi()
{
	if (match(";")==0) error("missing semicolon");
}

	/********************************/
	/*  Check for end of statement	*/
	/********************************/

needstend()
{
	skipblanks();
	return ((streq(ch,";")|(*ch==0)));
}

needbrack(str)
char *str;
{
	if (match(str)==0)
	{
		error("missing bracket");
		comment; outstr(str); nl;
	}
}

	/********************************/
	/*  Skip white space in input	*/
	/********************************/

skipblanks()
{
	while(1)
	{
		while (*ch==0)
		{
			inline();
			preprocess();
			if (eof) break;
		}
		if (*ch==' ') gch();
		else if (*ch==tabch) gch(); else return;
	}
}

	/********************************/
	/*   Skip this token and all	*/
	/*   suceeding white space	*/
	/********************************/

skipchars()
{
	if (an(inbyte())) while (an(*ch)) gch();
	else while (an(*ch)==0)
		{ if (*ch==0) break; gch(); }
	skipblanks();
}

	/********************************/
	/*     Test input character	*/
	/********************************/

alpha(c)
char c;
{
	if ((c>='a')&(c<='z')) return 1;
	if ((c>='A')&(c<='Z')) return 1;
	return (c=='_');
}

numeric(c)
char c;
{
	return ((c>='0')&(c<='9'));
}

an(c)
char c;
{
	if ((c>='0')&(c<='9')) return 1;
	if ((c>='a')&(c<='z')) return 1;
	if ((c>='A')&(c<='Z')) return 1;
	return (c=='_');
}

	/********************************/
	/* Get a character - read in a	*/
	/* line and preprocess if you	*/
	/*		have to		*/
	/********************************/

inbyte()
{
	while (*ch==0)
	{
		if (eof) return 0;
		inline();
		preprocess();
	}
	return gch();
}

	/********************************/
	/* Get a character - read in a	*/
	/*   line if you have to, but	*/
	/* don't preprocess (since this	*/
	/* is called from preprocessor!)*/
	/********************************/

inchar()
{
	if (*ch==0) inline();
	if (eof) return 0;
	return (gch());
}

	/********************************/
	/*	Read in a line		*/
	/********************************/

inline()
{
	int k, unit;

	while (1)
	{
		if (input==0)
			if (mulfile) openin(); else eof=1;
		if (eof) return;
		if ((unit=input2)==0) unit=input;
		resetptr();
		while ((k=getc(unit))>0)
		{
			if ((k=eolch)|(ch>=chmax))
				{lncnt++; break;}
			*ch++=k;
		}
		*ch=0;
		if (k<=0)
		{
			fclose(unit);
			if (input2) input2=0; else input=0;
		}
		if (ch>line)
		{
			if ((ctext)&(cmode))
			{
				comment;
				outstr(line);
				nl;
			}
			ch=line; nexch=ch+1;
			return;
		}
	}
}

	/********************************/
	/*  Preprocess a line - expand	*/
	/* macros and remove redundant	*/
	/*	tabs and spaces		*/
	/********************************/

preprocess()
{
	int k;
	char c,sname[namesize],*mq;

	if (cmode==0) return;
	mptr=mline;ch=line;nexch=ch+1;
	while (*ch)
	{
		if ((*ch==' ')|(*ch==tabch))
		{
			keepch(' ');
			while ((*ch==' ')|(*ch==tabch)) gch();
		}
		else if (*ch=='"')
		{
			keepch(*ch);
			gch();
			while(*ch!='"')
			{
				if (*ch==0)
					{error("missing quote");break;}
				keepch(gch());
			}
			gch();
			keepch('"');
		}
		else if (*ch==quoch)
		{
			keepch(quoch);
			gch();
			while(*ch!=quoch)
			{
				if (*ch==0)
					{error("missing apostrophe");break;}
				keepch(gch());
			}
			gch();
			keepch(quoch);
		}
		else if ((*ch=='/')&(*nexch=='*'))
		{
			inchar(); inchar();
			while (((*ch=='*')&(*nexch=='/'))==0)
			{
				if (*ch==0) inline(); else inchar();
				if (eof) break;
			}
			inchar(); inchar();
		}
		else if (an(*ch))
		{
			k=1;
			while (an(*ch))
			{
				if (k<namemax) sname[k++]=*ch;
				gch();
			}
			sname[k]=0; sname[0]=k-1;
			if (mq=findmac(sname))
				while (c=*mq++) keepch(c);
			else
			{
				k=1;
				while (c=sname[k++]) keepch(c);
			}
		}
		else keepch(gch());
	}
	keepch(0);
	if (mptr>=mpmax) error("line too long");
	mptr=mline; ch=line; nexch=ch+1;

	/* copy back to line buffer and strip parity for keeps */

	while (*ch++=*mptr++&127);
	ch=line;
}

	/********************************/
	/*	Reset input buffer	*/
	/********************************/

resetptr()
{
	ch=line;nexch=ch+1; *ch=0;
}

	/********************************/
	/*   Get next input character	*/
	/********************************/

gch()
{
	if (*ch==0) return 0;
	else {nexch++; return (*ch++);}
}

	/********************************/
	/* Save this character in buffer*/
	/********************************/

keepch(c)
char c;
{
	*mptr=c;
	if (mptr<mpmax) mptr++;
	return c;
}

	/********************************/
	/*    Get options from user	*/
	/********************************/

ask()
{
	resetptr();		/* clear input line */
	clearscreen;		/* clear the screen */
	display("        RatC : Lancaster implementation");
	display("        -------------------------------");
	nl;nl;
	glbflag=1; nxtlab=1; mulfile=0;
	display("Defaults:");
	display("       Globals defined   y");
	display("           First label   1");
	display("        Multiple files   n");
	nl;
	display("         Change defaults ? ");
	if (reply()) defaults();

	litlab=getlabel();	/* first label = literal pool */

		/* see if user wants to interleave c-text */
	ctext=0;
	display("        C-text to appear ? ");
	ctext=reply();

	resetptr();
}

defaults()
{
	int k,num[1];

		/* see if user wants us to allocate static */
		/* variables by name in this module	   */
		/* (pseudo external capability)		   */
	display("  Globals to be defined  ? ");
	glbflag=reply();

		/* get first allowable # for complier-generated */
		/* labels (so user can append modules)		*/
	while (1)
	{
		display("Starting number for labels ? ");
		gets(line);
		if (*ch==0) {num[0]=0; break;}
		if (k=number(num)) break;
	}
	nxtlab=num[0];

		/* find out if one input file only */
	display("    Multiple input files ? ");
	mulfile=reply();
}

	/********************************/
	/* Print a string to console	*/
	/********************************/

display(str)
char *str;
{
	int k;

	k=0;
	putchar(eolch);
	while (str[k]) putchar(str[k++]);
}

reply()
{
	resetptr();	/* clear input buffer */
	gets(line);
	if ((*ch=='Y')|(*ch=='y')) return 1; else return 0;
}

	/********************************/
	/*	Get output filename	*/
	/********************************/

openout()
{
	resetptr();		/* erase line */
	output=0;		/* start with none */
	display("      Output filename ? ");
	gets(line);
	if (*ch==0) return;	/* none given */
	if ((output=fopen(line,"w"))==NULL)
		{ output=0; error("Open failure"); }
	resetptr();
}

	/********************************/
	/*	Get (next) input file	*/
	/********************************/

openin()
{
	input=0;	/* none to start with */
	while (input==0)
	{
		resetptr();
		if (eof) break;
		display("    Input filename ? ");
		gets(line);
		if (*ch==0) {eof=1; break;}
		if ((input=fopen(line,"r"))==NULL)
			{input=0;display("Open failure");}
	}
	resetptr();
}

	/********************************/
	/*	Open an include file	*/
	/********************************/

openinclude()
{
	skipblanks();
	if ((input2=fopen(ch,"r"))==NULL)
		{input2=0; error("Open failure on include file"); }
	resetptr();
}

	/********************************/
	/*	Close the output file	*/
	/********************************/

closeout()
{
	if (output) fclose(output);
	output=0;
}

	/********************************/
	/*	Error reporting		*/
	/********************************/

errlval()
{
	error("must be lvalue");
}

errmulti(sname)
char *sname;
{
	error("already defined");
	comment;
	outstr(sname); nl;
}

errname()
{
	error("illegal symbol name"); skipchars();
}

	/********************************/
	/* Report type and posistion of	*/
	/* an error in the source line	*/
	/********************************/

error(ptr)
char ptr[];
{
	char *k;

	comment;outstr(line);nl;comment;
	k=line;
	while (k<ch)
	{
		if (*k==tabch) tab; else space;
		++k;
	}
	outbyte('^');
	nl;comment; outstr("------ ");
	outstr(ptr);
	outstr(" ------");
	display(line);display(ptr);	/* to console too */
	nl;
	++errcnt;
}

	/********************************/
	/*    Report errors for user	*/
	/********************************/

errorsummary()
{
	if (ncmp) error("missing closing bracket");
		/* open compound statement */
	nl;
	outdec(errcnt);		/* total # errors */
	outstr(" errors in compilation.");
	nl;

	nl;
	outdec(inscnt);	/* total # instructions */
	outstr(" instructions generated.");
	nl;

	nl;
	outdec(lncnt);	/* total # source lines */
	outstr(" source lines.");
	nl;
}

	/********************************/
	/*	Output a character	*/
	/********************************/

outbyte(c)
char c;
{
	if (c==0) return 0;
	if (output)
	{
		if ((putc(c,output))<=0)
			{closeout(); error("output file error");}
	}
	else putchar(c);
	return c;
}

	/********************************/
	/* Output a number in decimal	*/
	/********************************/

outdec(num)
int num;
{
	int k,zs;
	char c;

	zs=0;
	k=10000;
	if (num<0)
	{
		num=(-num);
		outbyte('-');
	}
	while (k>=1)
	{
		c=num/k+'0';
		if ((c!='0')|(k==1)|(zs)) {zs=1; outbyte(c);}
		num=num%k;
		k=k/10;
	}
}

	/********************************/
	/*	Output a string		*/
	/********************************/

outstr(ptr)
char ptr[];
{
	int k;

	/* ignore length byte if there is one */
	if (ptr[0]<32) k=1; else k=0;
	while (outbyte(ptr[k++]));
}

	/********************************/
	/*	Print a label		*/
	/********************************/

outlabel(label)
int label;
{
	outstr("cc");
	outdec(label);
}

	/********************************/
	/*	Instruction output	*/
	/********************************/

outline(ptr)
char ptr[];
	{outtab(ptr);nl;}

outtab(ptr)
char ptr[];
	{tab; outstr(ptr); inscnt++;}

	/********************************/
	/*    Evaluate condition	*/
	/********************************/

test(label)
int label;
{
	needbrack("(");
	expression();
	needbrack(")");
	testjump(label);
	nl;
}

	/********************************/
	/*     Store a value in memory	*/
	/********************************/

store(lval)
int *lval;
{
	if (lval[1]==0) putmem(lval[0]);
	else putstk(lval[0]);
}

	/********************************/
	/*    Get a value from memory	*/
	/********************************/

rvalue(lval)
int *lval;
{
	if ((lval[0]!=0)&(lval[1]==0)) getmem(lval[0]);
	else indirect(lval[1]);
}

	/********************************/
	/*   Load direct 8 or 16 bits	*/
	/*    into primary register	*/
	/********************************/

getmem(sym)
char *sym;
{
	if ((sym[ident]!=pointer)&(sym[type]==cchar))
		outtab("ldir.b");
	else outtab("ldir.w");
	outline(sym+name+1);
}

	/********************************/
	/* Given offset from SP, get	*/
	/* address into primary register*/
	/********************************/

getloc(sym)
char *sym;
{
	outtab("addr"); tab;
	outdec(((sym[offset]&255)+((sym[offset+1]&255)<<8))-sp);
	nl;
}

	/********************************/
	/*  Store direct 8 or 16 bits	*/
	/*    from primary register	*/
	/********************************/

putmem(sym)
char *sym;
{
	if ((sym[ident]!=pointer)&(sym[type]==cchar))
		outtab("sdir.b");
	else outtab("sdir.w");
	tab; outstr(sym+name);
	nl;
}

	/********************************/
	/* Store indirect 8 or 16 bits	*/
	/* at address on top of stack	*/
	/********************************/

putstk(typeobj)
char typeobj;
{
	pop();
	if (typeobj==cchar) outline("sind.b");
	else outline("sind.w");
}

	/********************************/
	/*  Load indirect 8 or 16 bits	*/
	/*  at address in primary reg	*/
	/*    into primary register	*/
	/********************************/

indirect(typeobj)
char typeobj;
{
	if (typeobj==cchar) outline("lind.b");
	else outline("lind.b");
}

	/********************************/
	/*	Call subroutine		*/
	/********************************/

call(sname)
char *sname;
{
	outtab("call");
	tab; outstr("_");outstr(sname);
	nl;
}

	/********************************/
	/*  Subroutine call to address	*/
	/*   on top of stack, return	*/
	/*    address left on stack	*/
	/********************************/

callstk()
{
	outline("scall");
	sp=sp+intwidth;
}

	/********************************/
	/*  Jump to specified internal	*/
	/*	  label number		*/
	/********************************/

jump(label)
int label;
{
	outtab("ujump");
	tab; outlabel(label);
	nl;
}

	/********************************/
	/* Jump to specified label if	*/
	/* primary reg is false (zero)	*/
	/********************************/

testjump(label)
int label;
{
	outtab("fjump");
	tab; outlabel(label);
}

	/********************************/
	/* Modify stack pointer to new	*/
	/*	 value indicated	*/
	/********************************/

modstk(newsp)
int newsp;
{
	int k;
	k=newsp-sp;
	if (k==0) return newsp;
	outtab("modstk");
	tab; outdec(k);
	nl;
	return newsp;
}

	/* start of main segment */
header()	{outline("start"); }
	/* swap primary and secondary */
swap()		{outline("swap");}
	/* paritial load immeadiate */
immed()		{outtab("limm"); tab;}
	/* push primary onto stack */
push()		{outline("push"); sp=sp-intwidth;}
	/* pop top of stack into secondary */
pop()		{outline("pop"); sp=sp+intwidth;}
	/* swap primary and top of stack */
swapstk()	{outline("xchange"); }
	/* return from subroutine */
ret()		{outline("return"); }

	/********************************/
	/*   Arithmetic and logical	*/
	/*  instructions - result in	*/
	/*      primary register	*/
	/********************************/

	/* scale primary by n */
scale(n)
int n;
{
	outtab("scale");
	tab; outdec(n); nl;
}
	/* add primary and secondary */
add()	{outline("add"); }
	/* subtract primary fro secondary */
sub()	{outline("sub"); }
	/* multiply primarry and secondary */
mult()	{outline("mult");}
	/* divide secondary by primary */
	/* quotient in primary, rem in secondary */
div()	{ outline("div"); }
	/* mod of secondary divided by primary */
	/* rem in primary, quotient in secondary */
mod()	{div(); swap(); }
	/* inclusive or of primary  and secondary */
or()	{outline("or"); }
	/* exclusive or of primary  and secondary */
xor()	{outline("xor"); }
	/* logical and  of primary  and secondary */
and()	{outline("and"); }
	/* arithmetic shift right of secondary */
	/* number of times in primary */
asr()	{outline("asr");}
	/* arithmetic shift left */
asl()	{outline("asl");}
	/* twos complement of primary */
neg()	{outline("neg");}
	/* increment primary by n */
inc(n)
int n;
{
	outtab("inc");
	tab; outdec(n); nl;
}
	/* decrement primary by n */
dec(n)
int n;
{
	outtab("dec");
	tab; outdec(n); nl;
}

	/********************************/
	/* Conditional instructions -	*/
	/* compare secondary against	*/
	/* primary, put 1 in primary if	*/
	/*	true, otherwise 0	*/
	/********************************/

	/* test for = */
eq()	{outline("testeq");}
	/* test for != */
ne()	{outline("testne");}
	/* test for < (signed) */
lt()	{outline("testlt");}
	/* test for <= (signed) */
le()	{outline("testle");}
	/* test for > (signed) */
gt()	{outline("testgt");}
	/* test for >= (signed) */
ge()	{outline("testge");}
	/* test for < (unsigned) */
ult()	{outline("testult");}
	/* test for <= (unsigned) */
ule()	{outline("testule");}
	/* test for > (unsigned) */
ugt()	{outline("testugt");}
	/* test for >= (unsigned) */
uge()	{outline("testuge");}

	/********************************/
	/*    Dump the literal pool	*/
	/********************************/

dumplits()
{
	int j,k;

	if (litptr==0) return;		/* if nothing there, exit */
	outlabel(litlab); colon;	/* print literal label */
	k=0;				/* init an index ... */
	while (k<litptr)		/* ... to loop with */
	{
		defbyte();		/* psuedo op to define byte */
		j=5;			/* max bytes per line */
		while (j--)
		{
			outdec((litq[k++]));
			if ((j==0)|(k>litptr))
				{ nl; break; }
			comma;		/* seperate bytes */
		}
	}
}

	/********************************/
	/*   Dump all static variables	*/
	/********************************/

dumpglbs()
{
	int j;

	if (glbflag==0) return;		/* dont if user said no */
	cptr=startglb;
	while (cptr<glbptr)
	{
		if (cptr[ident]!=function)
		/* do if anything but function */
		{
			outstr(cptr); colon;
					/* output name as label */
			defstorage();	/* define storage */
			j=((cptr[offset]&255)+
				((cptr[offset+1]&255)<<8));
					/* calculate # bytes */
			if ((cptr[type]=cint)|
				(cptr[ident]==pointer))
				j=j*intwidth;
			outdec(j);	/* need that many */
			nl;
		}
		cptr=cptr+symsiz;
	}
}

	/* literal definitions */
defbyte()	{ outtab("db "); }
defstorage()	{ outtab("ds "); }

	/* end of assembly */
trailer()
{
	outtab("end");
	if (hasmain) outtab("start");
	nl;
}


	/********************************/
	/*  <<< end of the compiler >>>	*/
	/********************************/
